@@ -1,8 +1,12 @@
 """Sensor platform for Advantage Air integration."""
 import voluptuous as vol
 
-from homeassistant.components.sensor import SensorEntity
-from homeassistant.const import PERCENTAGE
+from homeassistant.components.sensor import (
+    DEVICE_CLASS_TEMPERATURE,
+    STATE_CLASS_MEASUREMENT,
+    SensorEntity,
+)
+from homeassistant.const import ENTITY_CATEGORY_DIAGNOSTIC, PERCENTAGE, TEMP_CELSIUS
 from homeassistant.helpers import config_validation as cv, entity_platform
 
 from .const import ADVANTAGE_AIR_STATE_OPEN, DOMAIN as ADVANTAGE_AIR_DOMAIN
@@ -25,15 +29,16 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
         entities.append(AdvantageAirTimeTo(instance, ac_key, "On"))
         entities.append(AdvantageAirTimeTo(instance, ac_key, "Off"))
         for zone_key, zone in ac_device["zones"].items():
-            # Only show damper sensors when zone is in temperature control
+            # Only show damper and temp sensors when zone is in temperature control
             if zone["type"] != 0:
                 entities.append(AdvantageAirZoneVent(instance, ac_key, zone_key))
+                entities.append(AdvantageAirZoneTemp(instance, ac_key, zone_key))
             # Only show wireless signal strength sensors when using wireless sensors
             if zone["rssi"] > 0:
                 entities.append(AdvantageAirZoneSignal(instance, ac_key, zone_key))
     async_add_entities(entities)
 
-    platform = entity_platform.current_platform.get()
+    platform = entity_platform.async_get_current_platform()
     platform.async_register_entity_service(
         ADVANTAGE_AIR_SERVICE_SET_TIME_TO,
         {vol.Required("minutes"): cv.positive_int},
@@ -44,32 +49,24 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 class AdvantageAirTimeTo(AdvantageAirEntity, SensorEntity):
     """Representation of Advantage Air timer control."""
 
+    _attr_native_unit_of_measurement = ADVANTAGE_AIR_SET_COUNTDOWN_UNIT
+    _attr_entity_category = ENTITY_CATEGORY_DIAGNOSTIC
+
     def __init__(self, instance, ac_key, action):
         """Initialize the Advantage Air timer control."""
         super().__init__(instance, ac_key)
         self.action = action
-        self._time_key = f"countDownTo{self.action}"
+        self._time_key = f"countDownTo{action}"
+        self._attr_name = f'{self._ac["name"]} Time To {action}'
+        self._attr_unique_id = (
+            f'{self.coordinator.data["system"]["rid"]}-{self.ac_key}-timeto{action}'
+        )
 
     @property
-    def name(self):
-        """Return the name."""
-        return f'{self._ac["name"]} Time To {self.action}'
-
-    @property
-    def unique_id(self):
-        """Return a unique id."""
-        return f'{self.coordinator.data["system"]["rid"]}-{self.ac_key}-timeto{self.action}'
-
-    @property
-    def state(self):
+    def native_value(self):
         """Return the current value."""
         return self._ac[self._time_key]
 
-    @property
-    def unit_of_measurement(self):
-        """Return the unit of measurement."""
-        return ADVANTAGE_AIR_SET_COUNTDOWN_UNIT
-
     @property
     def icon(self):
         """Return a representative icon of the timer."""
@@ -86,28 +83,25 @@ class AdvantageAirTimeTo(AdvantageAirEntity, SensorEntity):
 class AdvantageAirZoneVent(AdvantageAirEntity, SensorEntity):
     """Representation of Advantage Air Zone Vent Sensor."""
 
-    @property
-    def name(self):
-        """Return the name."""
-        return f'{self._zone["name"]} Vent'
+    _attr_native_unit_of_measurement = PERCENTAGE
+    _attr_state_class = STATE_CLASS_MEASUREMENT
+    _attr_entity_category = ENTITY_CATEGORY_DIAGNOSTIC
+
+    def __init__(self, instance, ac_key, zone_key):
+        """Initialize an Advantage Air Zone Vent Sensor."""
+        super().__init__(instance, ac_key, zone_key=zone_key)
+        self._attr_name = f'{self._zone["name"]} Vent'
+        self._attr_unique_id = (
+            f'{self.coordinator.data["system"]["rid"]}-{ac_key}-{zone_key}-vent'
+        )
 
     @property
-    def unique_id(self):
-        """Return a unique id."""
-        return f'{self.coordinator.data["system"]["rid"]}-{self.ac_key}-{self.zone_key}-vent'
-
-    @property
-    def state(self):
+    def native_value(self):
         """Return the current value of the air vent."""
         if self._zone["state"] == ADVANTAGE_AIR_STATE_OPEN:
             return self._zone["value"]
         return 0
 
-    @property
-    def unit_of_measurement(self):
-        """Return the percent sign."""
-        return PERCENTAGE
-
     @property
     def icon(self):
         """Return a representative icon."""
@@ -119,26 +113,23 @@ class AdvantageAirZoneVent(AdvantageAirEntity, SensorEntity):
 class AdvantageAirZoneSignal(AdvantageAirEntity, SensorEntity):
     """Representation of Advantage Air Zone wireless signal sensor."""
 
-    @property
-    def name(self):
-        """Return the name."""
-        return f'{self._zone["name"]} Signal'
+    _attr_native_unit_of_measurement = PERCENTAGE
+    _attr_state_class = STATE_CLASS_MEASUREMENT
+    _attr_entity_category = ENTITY_CATEGORY_DIAGNOSTIC
+
+    def __init__(self, instance, ac_key, zone_key):
+        """Initialize an Advantage Air Zone wireless signal sensor."""
+        super().__init__(instance, ac_key, zone_key)
+        self._attr_name = f'{self._zone["name"]} Signal'
+        self._attr_unique_id = (
+            f'{self.coordinator.data["system"]["rid"]}-{ac_key}-{zone_key}-signal'
+        )
 
     @property
-    def unique_id(self):
-        """Return a unique id."""
-        return f'{self.coordinator.data["system"]["rid"]}-{self.ac_key}-{self.zone_key}-signal'
-
-    @property
-    def state(self):
+    def native_value(self):
         """Return the current value of the wireless signal."""
         return self._zone["rssi"]
 
-    @property
-    def unit_of_measurement(self):
-        """Return the percent sign."""
-        return PERCENTAGE
-
     @property
     def icon(self):
         """Return a representative icon."""
@@ -151,3 +142,26 @@ class AdvantageAirZoneSignal(AdvantageAirEntity, SensorEntity):
         if self._zone["rssi"] >= 20:
             return "mdi:wifi-strength-1"
         return "mdi:wifi-strength-outline"
+
+
+class AdvantageAirZoneTemp(AdvantageAirEntity, SensorEntity):
+    """Representation of Advantage Air Zone temperature sensor."""
+
+    _attr_native_unit_of_measurement = TEMP_CELSIUS
+    _attr_device_class = DEVICE_CLASS_TEMPERATURE
+    _attr_state_class = STATE_CLASS_MEASUREMENT
+    _attr_entity_registry_enabled_default = False
+    _attr_entity_category = ENTITY_CATEGORY_DIAGNOSTIC
+
+    def __init__(self, instance, ac_key, zone_key):
+        """Initialize an Advantage Air Zone Temp Sensor."""
+        super().__init__(instance, ac_key, zone_key)
+        self._attr_name = f'{self._zone["name"]} Temperature'
+        self._attr_unique_id = (
+            f'{self.coordinator.data["system"]["rid"]}-{ac_key}-{zone_key}-temp'
+        )
+
+    @property
+    def native_value(self):
+        """Return the current value of the measured temperature."""
+        return self._zone["measuredTemp"]
