@@ -1,7 +1,10 @@
 """The airvisual component."""
-import asyncio
+from __future__ import annotations
+
+from collections.abc import Mapping
 from datetime import timedelta
 from math import ceil
+from typing import Any, Dict, cast
 
 from pyairvisual import CloudAPI, NodeSamba
 from pyairvisual.errors import (
@@ -11,7 +14,7 @@ from pyairvisual.errors import (
     NodeProError,
 )
 
-from homeassistant.config_entries import SOURCE_REAUTH
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_ATTRIBUTION,
     CONF_API_KEY,
@@ -22,9 +25,14 @@ from homeassistant.const import (
     CONF_SHOW_ON_MAP,
     CONF_STATE,
 )
-from homeassistant.core import callback
-from homeassistant.exceptions import ConfigEntryNotReady
-from homeassistant.helpers import aiohttp_client, config_validation as cv
+from homeassistant.core import HomeAssistant, callback
+from homeassistant.exceptions import ConfigEntryAuthFailed
+from homeassistant.helpers import (
+    aiohttp_client,
+    config_validation as cv,
+    entity_registry,
+)
+from homeassistant.helpers.entity import EntityDescription
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
@@ -44,9 +52,7 @@ from .const import (
     LOGGER,
 )
 
-PLATFORMS = ["air_quality", "sensor"]
-
-DATA_LISTENER = "listener"
+PLATFORMS = ["sensor"]
 
 DEFAULT_ATTRIBUTION = "Data provided by AirVisual"
 DEFAULT_NODE_PRO_UPDATE_INTERVAL = timedelta(minutes=1)
@@ -55,11 +61,8 @@ CONFIG_SCHEMA = cv.deprecated(DOMAIN)
 
 
 @callback
-def async_get_geography_id(geography_dict):
+def async_get_geography_id(geography_dict: Mapping[str, Any]) -> str:
     """Generate a unique ID from a geography dict."""
-    if not geography_dict:
-        return
-
     if CONF_CITY in geography_dict:
         return ", ".join(
             (
@@ -74,15 +77,17 @@ def async_get_geography_id(geography_dict):
 
 
 @callback
-def async_get_cloud_api_update_interval(hass, api_key, num_consumers):
+def async_get_cloud_api_update_interval(
+    hass: HomeAssistant, api_key: str, num_consumers: int
+) -> timedelta:
     """Get a leveled scan interval for a particular cloud API key.
 
     This will shift based on the number of active consumers, thus keeping the user
     under the monthly API limit.
     """
-    # Assuming 10,000 calls per month and a "smallest possible month" of 28 days; note
+    # Assuming 10,000 calls per month and a "largest possible month" of 31 days; note
     # that we give a buffer of 1500 API calls for any drift, restarts, etc.:
-    minutes_between_api_calls = ceil(1 / (8500 / 28 / 24 / 60 / num_consumers))
+    minutes_between_api_calls = ceil(num_consumers * 31 * 24 * 60 / 8500)
 
     LOGGER.debug(
         "Leveling API key usage (%s): %s consumers, %s minutes between updates",
@@ -95,18 +100,23 @@ def async_get_cloud_api_update_interval(hass, api_key, num_consumers):
 
 
 @callback
-def async_get_cloud_coordinators_by_api_key(hass, api_key):
+def async_get_cloud_coordinators_by_api_key(
+    hass: HomeAssistant, api_key: str
+) -> list[DataUpdateCoordinator]:
     """Get all DataUpdateCoordinator objects related to a particular API key."""
-    coordinators = []
-    for entry_id, coordinator in hass.data[DOMAIN][DATA_COORDINATOR].items():
-        config_entry = hass.config_entries.async_get_entry(entry_id)
-        if config_entry.data.get(CONF_API_KEY) == api_key:
-            coordinators.append(coordinator)
-    return coordinators
+    return [
+        coordinator
+        for entry_id, attrs in hass.data[DOMAIN].items()
+        if (entry := hass.config_entries.async_get_entry(entry_id))
+        and (coordinator := attrs.get(DATA_COORDINATOR))
+        and entry.data.get(CONF_API_KEY) == api_key
+    ]
 
 
 @callback
-def async_sync_geo_coordinator_update_intervals(hass, api_key):
+def async_sync_geo_coordinator_update_intervals(
+    hass: HomeAssistant, api_key: str
+) -> None:
     """Sync the update interval for geography-based data coordinators (by API key)."""
     coordinators = async_get_cloud_coordinators_by_api_key(hass, api_key)
 
@@ -126,31 +136,27 @@ def async_sync_geo_coordinator_update_intervals(hass, api_key):
         coordinator.update_interval = update_interval
 
 
-async def async_setup(hass, config):
-    """Set up the AirVisual component."""
-    hass.data[DOMAIN] = {DATA_COORDINATOR: {}, DATA_LISTENER: {}}
-    return True
-
-
 @callback
-def _standardize_geography_config_entry(hass, config_entry):
+def _standardize_geography_config_entry(
+    hass: HomeAssistant, entry: ConfigEntry
+) -> None:
     """Ensure that geography config entries have appropriate properties."""
     entry_updates = {}
 
-    if not config_entry.unique_id:
+    if not entry.unique_id:
         # If the config entry doesn't already have a unique ID, set one:
-        entry_updates["unique_id"] = config_entry.data[CONF_API_KEY]
-    if not config_entry.options:
+        entry_updates["unique_id"] = entry.data[CONF_API_KEY]
+    if not entry.options:
         # If the config entry doesn't already have any options set, set defaults:
         entry_updates["options"] = {CONF_SHOW_ON_MAP: True}
-    if config_entry.data.get(CONF_INTEGRATION_TYPE) not in [
+    if entry.data.get(CONF_INTEGRATION_TYPE) not in [
         INTEGRATION_TYPE_GEOGRAPHY_COORDS,
         INTEGRATION_TYPE_GEOGRAPHY_NAME,
     ]:
         # If the config entry data doesn't contain an integration type that we know
         # about, infer it from the data we have:
-        entry_updates["data"] = {**config_entry.data}
-        if CONF_CITY in config_entry.data:
+        entry_updates["data"] = {**entry.data}
+        if CONF_CITY in entry.data:
             entry_updates["data"][
                 CONF_INTEGRATION_TYPE
             ] = INTEGRATION_TYPE_GEOGRAPHY_NAME
@@ -162,79 +168,64 @@ def _standardize_geography_config_entry(hass, config_entry):
     if not entry_updates:
         return
 
-    hass.config_entries.async_update_entry(config_entry, **entry_updates)
+    hass.config_entries.async_update_entry(entry, **entry_updates)
 
 
 @callback
-def _standardize_node_pro_config_entry(hass, config_entry):
+def _standardize_node_pro_config_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
     """Ensure that Node/Pro config entries have appropriate properties."""
-    entry_updates = {}
+    entry_updates: dict[str, Any] = {}
 
-    if CONF_INTEGRATION_TYPE not in config_entry.data:
+    if CONF_INTEGRATION_TYPE not in entry.data:
         # If the config entry data doesn't contain the integration type, add it:
         entry_updates["data"] = {
-            **config_entry.data,
+            **entry.data,
             CONF_INTEGRATION_TYPE: INTEGRATION_TYPE_NODE_PRO,
         }
 
     if not entry_updates:
         return
 
-    hass.config_entries.async_update_entry(config_entry, **entry_updates)
+    hass.config_entries.async_update_entry(entry, **entry_updates)
 
 
-async def async_setup_entry(hass, config_entry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up AirVisual as config entry."""
-    if CONF_API_KEY in config_entry.data:
-        _standardize_geography_config_entry(hass, config_entry)
+    hass.data.setdefault(DOMAIN, {})
+    hass.data[DOMAIN][entry.entry_id] = {}
+
+    if CONF_API_KEY in entry.data:
+        _standardize_geography_config_entry(hass, entry)
 
         websession = aiohttp_client.async_get_clientsession(hass)
-        cloud_api = CloudAPI(config_entry.data[CONF_API_KEY], session=websession)
+        cloud_api = CloudAPI(entry.data[CONF_API_KEY], session=websession)
 
-        async def async_update_data():
+        async def async_update_data() -> dict[str, Any]:
             """Get new data from the API."""
-            if CONF_CITY in config_entry.data:
+            if CONF_CITY in entry.data:
                 api_coro = cloud_api.air_quality.city(
-                    config_entry.data[CONF_CITY],
-                    config_entry.data[CONF_STATE],
-                    config_entry.data[CONF_COUNTRY],
+                    entry.data[CONF_CITY],
+                    entry.data[CONF_STATE],
+                    entry.data[CONF_COUNTRY],
                 )
             else:
                 api_coro = cloud_api.air_quality.nearest_city(
-                    config_entry.data[CONF_LATITUDE],
-                    config_entry.data[CONF_LONGITUDE],
+                    entry.data[CONF_LATITUDE],
+                    entry.data[CONF_LONGITUDE],
                 )
 
             try:
-                return await api_coro
-            except (InvalidKeyError, KeyExpiredError):
-                matching_flows = [
-                    flow
-                    for flow in hass.config_entries.flow.async_progress()
-                    if flow["context"]["source"] == SOURCE_REAUTH
-                    and flow["context"]["unique_id"] == config_entry.unique_id
-                ]
-
-                if not matching_flows:
-                    hass.async_create_task(
-                        hass.config_entries.flow.async_init(
-                            DOMAIN,
-                            context={
-                                "source": SOURCE_REAUTH,
-                                "unique_id": config_entry.unique_id,
-                            },
-                            data=config_entry.data,
-                        )
-                    )
-
-                return {}
+                data = await api_coro
+                return cast(Dict[str, Any], data)
+            except (InvalidKeyError, KeyExpiredError) as ex:
+                raise ConfigEntryAuthFailed from ex
             except AirVisualError as err:
                 raise UpdateFailed(f"Error while retrieving data: {err}") from err
 
         coordinator = DataUpdateCoordinator(
             hass,
             LOGGER,
-            name=async_get_geography_id(config_entry.data),
+            name=async_get_geography_id(entry.data),
             # We give a placeholder update interval in order to create the coordinator;
             # then, below, we use the coordinator's presence (along with any other
             # coordinators using the same API key) to calculate an actual, leveled
@@ -243,24 +234,32 @@ async def async_setup_entry(hass, config_entry):
             update_method=async_update_data,
         )
 
-        async_sync_geo_coordinator_update_intervals(
-            hass, config_entry.data[CONF_API_KEY]
-        )
-
         # Only geography-based entries have options:
-        hass.data[DOMAIN][DATA_LISTENER][
-            config_entry.entry_id
-        ] = config_entry.add_update_listener(async_reload_entry)
+        entry.async_on_unload(entry.add_update_listener(async_reload_entry))
     else:
-        _standardize_node_pro_config_entry(hass, config_entry)
+        # Remove outdated air_quality entities from the entity registry if they exist:
+        ent_reg = entity_registry.async_get(hass)
+        for entity_entry in [
+            e
+            for e in ent_reg.entities.values()
+            if e.config_entry_id == entry.entry_id
+            and e.entity_id.startswith("air_quality")
+        ]:
+            LOGGER.debug(
+                'Removing deprecated air_quality entity: "%s"', entity_entry.entity_id
+            )
+            ent_reg.async_remove(entity_entry.entity_id)
 
-        async def async_update_data():
+        _standardize_node_pro_config_entry(hass, entry)
+
+        async def async_update_data() -> dict[str, Any]:
             """Get new data from the API."""
             try:
                 async with NodeSamba(
-                    config_entry.data[CONF_IP_ADDRESS], config_entry.data[CONF_PASSWORD]
+                    entry.data[CONF_IP_ADDRESS], entry.data[CONF_PASSWORD]
                 ) as node:
-                    return await node.async_get_latest_measurements()
+                    data = await node.async_get_latest_measurements()
+                    return cast(Dict[str, Any], data)
             except NodeProError as err:
                 raise UpdateFailed(f"Error while retrieving data: {err}") from err
 
@@ -272,41 +271,39 @@ async def async_setup_entry(hass, config_entry):
             update_method=async_update_data,
         )
 
-    await coordinator.async_refresh()
-    if not coordinator.last_update_success:
-        raise ConfigEntryNotReady
+    await coordinator.async_config_entry_first_refresh()
+    hass.data[DOMAIN][entry.entry_id][DATA_COORDINATOR] = coordinator
 
-    hass.data[DOMAIN][DATA_COORDINATOR][config_entry.entry_id] = coordinator
+    # Reassess the interval between 2 server requests
+    if CONF_API_KEY in entry.data:
+        async_sync_geo_coordinator_update_intervals(hass, entry.data[CONF_API_KEY])
 
-    for component in PLATFORMS:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(config_entry, component)
-        )
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
 
-async def async_migrate_entry(hass, config_entry):
+async def async_migrate_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Migrate an old config entry."""
-    version = config_entry.version
+    version = entry.version
 
     LOGGER.debug("Migrating from version %s", version)
 
     # 1 -> 2: One geography per config entry
     if version == 1:
-        version = config_entry.version = 2
+        version = entry.version = 2
 
         # Update the config entry to only include the first geography (there is always
         # guaranteed to be at least one):
-        geographies = list(config_entry.data[CONF_GEOGRAPHIES])
+        geographies = list(entry.data[CONF_GEOGRAPHIES])
         first_geography = geographies.pop(0)
         first_id = async_get_geography_id(first_geography)
 
         hass.config_entries.async_update_entry(
-            config_entry,
+            entry,
             unique_id=first_id,
             title=f"Cloud API ({first_id})",
-            data={CONF_API_KEY: config_entry.data[CONF_API_KEY], **first_geography},
+            data={CONF_API_KEY: entry.data[CONF_API_KEY], **first_geography},
         )
 
         # For any geographies that remain, create a new config entry for each one:
@@ -319,7 +316,7 @@ async def async_migrate_entry(hass, config_entry):
                 hass.config_entries.flow.async_init(
                     DOMAIN,
                     context={"source": source},
-                    data={CONF_API_KEY: config_entry.data[CONF_API_KEY], **geography},
+                    data={CONF_API_KEY: entry.data[CONF_API_KEY], **geography},
                 )
             )
 
@@ -328,69 +325,46 @@ async def async_migrate_entry(hass, config_entry):
     return True
 
 
-async def async_unload_entry(hass, config_entry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload an AirVisual config entry."""
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(config_entry, component)
-                for component in PLATFORMS
-            ]
-        )
-    )
-    if unload_ok:
-        hass.data[DOMAIN][DATA_COORDINATOR].pop(config_entry.entry_id)
-        remove_listener = hass.data[DOMAIN][DATA_LISTENER].pop(config_entry.entry_id)
-        remove_listener()
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
-        if (
-            config_entry.data[CONF_INTEGRATION_TYPE]
-            == INTEGRATION_TYPE_GEOGRAPHY_COORDS
-        ):
+    if unload_ok:
+        hass.data[DOMAIN].pop(entry.entry_id)
+        if CONF_API_KEY in entry.data:
             # Re-calculate the update interval period for any remaining consumers of
             # this API key:
-            async_sync_geo_coordinator_update_intervals(
-                hass, config_entry.data[CONF_API_KEY]
-            )
+            async_sync_geo_coordinator_update_intervals(hass, entry.data[CONF_API_KEY])
 
     return unload_ok
 
 
-async def async_reload_entry(hass, config_entry):
+async def async_reload_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
     """Handle an options update."""
-    await hass.config_entries.async_reload(config_entry.entry_id)
+    await hass.config_entries.async_reload(entry.entry_id)
 
 
 class AirVisualEntity(CoordinatorEntity):
     """Define a generic AirVisual entity."""
 
-    def __init__(self, coordinator):
+    def __init__(
+        self,
+        coordinator: DataUpdateCoordinator,
+        entry: ConfigEntry,
+        description: EntityDescription,
+    ) -> None:
         """Initialize."""
         super().__init__(coordinator)
-        self._attrs = {ATTR_ATTRIBUTION: DEFAULT_ATTRIBUTION}
-        self._icon = None
-        self._unit = None
 
-    @property
-    def device_state_attributes(self):
-        """Return the device state attributes."""
-        return self._attrs
+        self._attr_extra_state_attributes = {ATTR_ATTRIBUTION: DEFAULT_ATTRIBUTION}
+        self._entry = entry
+        self.entity_description = description
 
-    @property
-    def icon(self):
-        """Return the icon."""
-        return self._icon
-
-    @property
-    def unit_of_measurement(self):
-        """Return the unit the value is expressed in."""
-        return self._unit
-
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Register callbacks."""
 
         @callback
-        def update():
+        def update() -> None:
             """Update the state."""
             self.update_from_latest_data()
             self.async_write_ha_state()
@@ -400,6 +374,6 @@ class AirVisualEntity(CoordinatorEntity):
         self.update_from_latest_data()
 
     @callback
-    def update_from_latest_data(self):
+    def update_from_latest_data(self) -> None:
         """Update the entity from the latest data."""
         raise NotImplementedError
