@@ -1,7 +1,27 @@
 """Support for deCONZ climate devices."""
-from typing import Optional
+from __future__ import annotations
 
-from pydeconz.sensor import Thermostat
+from pydeconz.sensor import (
+    THERMOSTAT_FAN_MODE_AUTO,
+    THERMOSTAT_FAN_MODE_HIGH,
+    THERMOSTAT_FAN_MODE_LOW,
+    THERMOSTAT_FAN_MODE_MEDIUM,
+    THERMOSTAT_FAN_MODE_OFF,
+    THERMOSTAT_FAN_MODE_ON,
+    THERMOSTAT_FAN_MODE_SMART,
+    THERMOSTAT_MODE_AUTO,
+    THERMOSTAT_MODE_COOL,
+    THERMOSTAT_MODE_HEAT,
+    THERMOSTAT_MODE_OFF,
+    THERMOSTAT_PRESET_AUTO,
+    THERMOSTAT_PRESET_BOOST,
+    THERMOSTAT_PRESET_COMFORT,
+    THERMOSTAT_PRESET_COMPLEX,
+    THERMOSTAT_PRESET_ECO,
+    THERMOSTAT_PRESET_HOLIDAY,
+    THERMOSTAT_PRESET_MANUAL,
+    Thermostat,
+)
 
 from homeassistant.components.climate import DOMAIN, ClimateEntity
 from homeassistant.components.climate.const import (
@@ -26,29 +46,29 @@ from homeassistant.const import ATTR_TEMPERATURE, TEMP_CELSIUS
 from homeassistant.core import callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 
-from .const import ATTR_LOCKED, ATTR_OFFSET, ATTR_VALVE, NEW_SENSOR
+from .const import ATTR_LOCKED, ATTR_OFFSET, ATTR_VALVE
 from .deconz_device import DeconzDevice
 from .gateway import get_gateway_from_config_entry
 
 DECONZ_FAN_SMART = "smart"
 
 FAN_MODE_TO_DECONZ = {
-    DECONZ_FAN_SMART: "smart",
-    FAN_AUTO: "auto",
-    FAN_HIGH: "high",
-    FAN_MEDIUM: "medium",
-    FAN_LOW: "low",
-    FAN_ON: "on",
-    FAN_OFF: "off",
+    DECONZ_FAN_SMART: THERMOSTAT_FAN_MODE_SMART,
+    FAN_AUTO: THERMOSTAT_FAN_MODE_AUTO,
+    FAN_HIGH: THERMOSTAT_FAN_MODE_HIGH,
+    FAN_MEDIUM: THERMOSTAT_FAN_MODE_MEDIUM,
+    FAN_LOW: THERMOSTAT_FAN_MODE_LOW,
+    FAN_ON: THERMOSTAT_FAN_MODE_ON,
+    FAN_OFF: THERMOSTAT_FAN_MODE_OFF,
 }
 
 DECONZ_TO_FAN_MODE = {value: key for key, value in FAN_MODE_TO_DECONZ.items()}
 
 HVAC_MODE_TO_DECONZ = {
-    HVAC_MODE_AUTO: "auto",
-    HVAC_MODE_COOL: "cool",
-    HVAC_MODE_HEAT: "heat",
-    HVAC_MODE_OFF: "off",
+    HVAC_MODE_AUTO: THERMOSTAT_MODE_AUTO,
+    HVAC_MODE_COOL: THERMOSTAT_MODE_COOL,
+    HVAC_MODE_HEAT: THERMOSTAT_MODE_HEAT,
+    HVAC_MODE_OFF: THERMOSTAT_MODE_OFF,
 }
 
 DECONZ_PRESET_AUTO = "auto"
@@ -57,13 +77,13 @@ DECONZ_PRESET_HOLIDAY = "holiday"
 DECONZ_PRESET_MANUAL = "manual"
 
 PRESET_MODE_TO_DECONZ = {
-    DECONZ_PRESET_AUTO: "auto",
-    PRESET_BOOST: "boost",
-    PRESET_COMFORT: "comfort",
-    DECONZ_PRESET_COMPLEX: "complex",
-    PRESET_ECO: "eco",
-    DECONZ_PRESET_HOLIDAY: "holiday",
-    DECONZ_PRESET_MANUAL: "manual",
+    DECONZ_PRESET_AUTO: THERMOSTAT_PRESET_AUTO,
+    PRESET_BOOST: THERMOSTAT_PRESET_BOOST,
+    PRESET_COMFORT: THERMOSTAT_PRESET_COMFORT,
+    DECONZ_PRESET_COMPLEX: THERMOSTAT_PRESET_COMPLEX,
+    PRESET_ECO: THERMOSTAT_PRESET_ECO,
+    DECONZ_PRESET_HOLIDAY: THERMOSTAT_PRESET_HOLIDAY,
+    DECONZ_PRESET_MANUAL: THERMOSTAT_PRESET_MANUAL,
 }
 
 DECONZ_TO_PRESET_MODE = {value: key for key, value in PRESET_MODE_TO_DECONZ.items()}
@@ -84,22 +104,23 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 
         for sensor in sensors:
 
+            if not gateway.option_allow_clip_sensor and sensor.type.startswith("CLIP"):
+                continue
+
             if (
-                sensor.type in Thermostat.ZHATYPE
-                and sensor.uniqueid not in gateway.entities[DOMAIN]
-                and (
-                    gateway.option_allow_clip_sensor
-                    or not sensor.type.startswith("CLIP")
-                )
+                isinstance(sensor, Thermostat)
+                and sensor.unique_id not in gateway.entities[DOMAIN]
             ):
                 entities.append(DeconzThermostat(sensor, gateway))
 
         if entities:
             async_add_entities(entities)
 
-    gateway.listeners.append(
+    config_entry.async_on_unload(
         async_dispatcher_connect(
-            hass, gateway.async_signal_new_device(NEW_SENSOR), async_add_climate
+            hass,
+            gateway.signal_new_sensor,
+            async_add_climate,
         )
     )
 
@@ -110,13 +131,14 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
     """Representation of a deCONZ thermostat."""
 
     TYPE = DOMAIN
+    _attr_temperature_unit = TEMP_CELSIUS
 
     def __init__(self, device, gateway):
         """Set up thermostat device."""
         super().__init__(device, gateway)
 
         self._hvac_mode_to_deconz = dict(HVAC_MODE_TO_DECONZ)
-        if "mode" not in device.raw["config"]:
+        if not device.mode:
             self._hvac_mode_to_deconz = {
                 HVAC_MODE_HEAT: True,
                 HVAC_MODE_OFF: False,
@@ -127,18 +149,13 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
             value: key for key, value in self._hvac_mode_to_deconz.items()
         }
 
-        self._features = SUPPORT_TARGET_TEMPERATURE
+        self._attr_supported_features = SUPPORT_TARGET_TEMPERATURE
 
-        if "fanmode" in device.raw["config"]:
-            self._features |= SUPPORT_FAN_MODE
+        if device.fan_mode:
+            self._attr_supported_features |= SUPPORT_FAN_MODE
 
-        if "preset" in device.raw["config"]:
-            self._features |= SUPPORT_PRESET_MODE
-
-    @property
-    def supported_features(self):
-        """Return the list of supported features."""
-        return self._features
+        if device.preset:
+            self._attr_supported_features |= SUPPORT_PRESET_MODE
 
     # Fan control
 
@@ -146,7 +163,7 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
     def fan_mode(self) -> str:
         """Return fan operation."""
         return DECONZ_TO_FAN_MODE.get(
-            self._device.fanmode, FAN_ON if self._device.state_on else FAN_OFF
+            self._device.fan_mode, FAN_ON if self._device.state_on else FAN_OFF
         )
 
     @property
@@ -159,9 +176,7 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
         if fan_mode not in FAN_MODE_TO_DECONZ:
             raise ValueError(f"Unsupported fan mode {fan_mode}")
 
-        data = {"fanmode": FAN_MODE_TO_DECONZ[fan_mode]}
-
-        await self._device.async_set_config(data)
+        await self._device.set_config(fan_mode=FAN_MODE_TO_DECONZ[fan_mode])
 
     # HVAC control
 
@@ -190,12 +205,12 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
         if len(self._hvac_mode_to_deconz) == 2:  # Only allow turn on and off thermostat
             data = {"on": self._hvac_mode_to_deconz[hvac_mode]}
 
-        await self._device.async_set_config(data)
+        await self._device.set_config(**data)
 
     # Preset control
 
     @property
-    def preset_mode(self) -> Optional[str]:
+    def preset_mode(self) -> str | None:
         """Return preset mode."""
         return DECONZ_TO_PRESET_MODE.get(self._device.preset)
 
@@ -209,9 +224,7 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
         if preset_mode not in PRESET_MODE_TO_DECONZ:
             raise ValueError(f"Unsupported preset mode {preset_mode}")
 
-        data = {"preset": PRESET_MODE_TO_DECONZ[preset_mode]}
-
-        await self._device.async_set_config(data)
+        await self._device.set_config(preset=PRESET_MODE_TO_DECONZ[preset_mode])
 
     # Temperature control
 
@@ -223,28 +236,23 @@ class DeconzThermostat(DeconzDevice, ClimateEntity):
     @property
     def target_temperature(self) -> float:
         """Return the target temperature."""
-        if self._device.mode == "cool":
-            return self._device.coolsetpoint
-        return self._device.heatsetpoint
+        if self._device.mode == THERMOSTAT_MODE_COOL:
+            return self._device.cooling_setpoint
+        return self._device.heating_setpoint
 
     async def async_set_temperature(self, **kwargs):
         """Set new target temperature."""
         if ATTR_TEMPERATURE not in kwargs:
             raise ValueError(f"Expected attribute {ATTR_TEMPERATURE}")
 
-        data = {"heatsetpoint": kwargs[ATTR_TEMPERATURE] * 100}
+        data = {"heating_setpoint": kwargs[ATTR_TEMPERATURE] * 100}
         if self._device.mode == "cool":
-            data = {"coolsetpoint": kwargs[ATTR_TEMPERATURE] * 100}
+            data = {"cooling_setpoint": kwargs[ATTR_TEMPERATURE] * 100}
 
-        await self._device.async_set_config(data)
+        await self._device.set_config(**data)
 
     @property
-    def temperature_unit(self):
-        """Return the unit of measurement."""
-        return TEMP_CELSIUS
-
-    @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes of the thermostat."""
         attr = {}
 
