@@ -1,10 +1,10 @@
 """Support for deCONZ sensors."""
 from pydeconz.sensor import (
-    AncillaryControl,
+    AirQuality,
     Battery,
     Consumption,
     Daylight,
-    DoorLock,
+    GenericStatus,
     Humidity,
     LightLevel,
     Power,
@@ -12,23 +12,28 @@ from pydeconz.sensor import (
     Switch,
     Temperature,
     Thermostat,
+    Time,
 )
 
 from homeassistant.components.sensor import (
     DOMAIN,
     STATE_CLASS_MEASUREMENT,
+    STATE_CLASS_TOTAL_INCREASING,
     SensorEntity,
+    SensorEntityDescription,
 )
 from homeassistant.const import (
     ATTR_TEMPERATURE,
     ATTR_VOLTAGE,
     DEVICE_CLASS_BATTERY,
+    DEVICE_CLASS_ENERGY,
     DEVICE_CLASS_HUMIDITY,
     DEVICE_CLASS_ILLUMINANCE,
     DEVICE_CLASS_POWER,
     DEVICE_CLASS_PRESSURE,
     DEVICE_CLASS_TEMPERATURE,
     ENERGY_KILO_WATT_HOUR,
+    ENTITY_CATEGORY_DIAGNOSTIC,
     LIGHT_LUX,
     PERCENTAGE,
     POWER_WATT,
@@ -41,42 +46,76 @@ from homeassistant.helpers.dispatcher import (
     async_dispatcher_send,
 )
 
-from .const import ATTR_DARK, ATTR_ON, NEW_SENSOR
+from .const import ATTR_DARK, ATTR_ON
 from .deconz_device import DeconzDevice
 from .gateway import get_gateway_from_config_entry
 
+DECONZ_SENSORS = (
+    AirQuality,
+    Consumption,
+    Daylight,
+    GenericStatus,
+    Humidity,
+    LightLevel,
+    Power,
+    Pressure,
+    Temperature,
+    Time,
+)
+
 ATTR_CURRENT = "current"
 ATTR_POWER = "power"
 ATTR_DAYLIGHT = "daylight"
 ATTR_EVENT_ID = "event_id"
 
-DEVICE_CLASS = {
-    Humidity: DEVICE_CLASS_HUMIDITY,
-    LightLevel: DEVICE_CLASS_ILLUMINANCE,
-    Power: DEVICE_CLASS_POWER,
-    Pressure: DEVICE_CLASS_PRESSURE,
-    Temperature: DEVICE_CLASS_TEMPERATURE,
-}
-
-ICON = {
-    Daylight: "mdi:white-balance-sunny",
-    Pressure: "mdi:gauge",
-    Temperature: "mdi:thermometer",
-}
-
-STATE_CLASS = {
-    Humidity: STATE_CLASS_MEASUREMENT,
-    Pressure: STATE_CLASS_MEASUREMENT,
-    Temperature: STATE_CLASS_MEASUREMENT,
-}
-
-UNIT_OF_MEASUREMENT = {
-    Consumption: ENERGY_KILO_WATT_HOUR,
-    Humidity: PERCENTAGE,
-    LightLevel: LIGHT_LUX,
-    Power: POWER_WATT,
-    Pressure: PRESSURE_HPA,
-    Temperature: TEMP_CELSIUS,
+ENTITY_DESCRIPTIONS = {
+    Battery: SensorEntityDescription(
+        key="battery",
+        device_class=DEVICE_CLASS_BATTERY,
+        state_class=STATE_CLASS_MEASUREMENT,
+        native_unit_of_measurement=PERCENTAGE,
+        entity_category=ENTITY_CATEGORY_DIAGNOSTIC,
+    ),
+    Consumption: SensorEntityDescription(
+        key="consumption",
+        device_class=DEVICE_CLASS_ENERGY,
+        state_class=STATE_CLASS_TOTAL_INCREASING,
+        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
+    ),
+    Daylight: SensorEntityDescription(
+        key="daylight",
+        icon="mdi:white-balance-sunny",
+        entity_registry_enabled_default=False,
+    ),
+    Humidity: SensorEntityDescription(
+        key="humidity",
+        device_class=DEVICE_CLASS_HUMIDITY,
+        state_class=STATE_CLASS_MEASUREMENT,
+        native_unit_of_measurement=PERCENTAGE,
+    ),
+    LightLevel: SensorEntityDescription(
+        key="lightlevel",
+        device_class=DEVICE_CLASS_ILLUMINANCE,
+        native_unit_of_measurement=LIGHT_LUX,
+    ),
+    Power: SensorEntityDescription(
+        key="power",
+        device_class=DEVICE_CLASS_POWER,
+        state_class=STATE_CLASS_MEASUREMENT,
+        native_unit_of_measurement=POWER_WATT,
+    ),
+    Pressure: SensorEntityDescription(
+        key="pressure",
+        device_class=DEVICE_CLASS_PRESSURE,
+        state_class=STATE_CLASS_MEASUREMENT,
+        native_unit_of_measurement=PRESSURE_HPA,
+    ),
+    Temperature: SensorEntityDescription(
+        key="temperature",
+        device_class=DEVICE_CLASS_TEMPERATURE,
+        state_class=STATE_CLASS_MEASUREMENT,
+        native_unit_of_measurement=TEMP_CELSIUS,
+    ),
 }
 
 
@@ -113,14 +152,9 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                 battery_handler.create_tracker(sensor)
 
             if (
-                not sensor.BINARY
-                and sensor.type
-                not in AncillaryControl.ZHATYPE
-                + Battery.ZHATYPE
-                + DoorLock.ZHATYPE
-                + Switch.ZHATYPE
-                + Thermostat.ZHATYPE
-                and sensor.uniqueid not in gateway.entities[DOMAIN]
+                isinstance(sensor, DECONZ_SENSORS)
+                and not isinstance(sensor, Thermostat)
+                and sensor.unique_id not in gateway.entities[DOMAIN]
             ):
                 entities.append(DeconzSensor(sensor, gateway))
 
@@ -135,7 +169,9 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 
     config_entry.async_on_unload(
         async_dispatcher_connect(
-            hass, gateway.async_signal_new_device(NEW_SENSOR), async_add_sensor
+            hass,
+            gateway.signal_new_sensor,
+            async_add_sensor,
         )
     )
 
@@ -153,20 +189,18 @@ class DeconzSensor(DeconzDevice, SensorEntity):
         """Initialize deCONZ binary sensor."""
         super().__init__(device, gateway)
 
-        self._attr_device_class = DEVICE_CLASS.get(type(self._device))
-        self._attr_icon = ICON.get(type(self._device))
-        self._attr_state_class = STATE_CLASS.get(type(self._device))
-        self._attr_unit_of_measurement = UNIT_OF_MEASUREMENT.get(type(self._device))
+        if entity_description := ENTITY_DESCRIPTIONS.get(type(device)):
+            self.entity_description = entity_description
 
     @callback
-    def async_update_callback(self, force_update=False):
+    def async_update_callback(self):
         """Update the sensor's state."""
         keys = {"on", "reachable", "state"}
-        if force_update or self._device.changed_keys.intersection(keys):
-            super().async_update_callback(force_update=force_update)
+        if self._device.changed_keys.intersection(keys):
+            super().async_update_callback()
 
     @property
-    def state(self):
+    def native_value(self):
         """Return the state of the sensor."""
         return self._device.state
 
@@ -181,13 +215,13 @@ class DeconzSensor(DeconzDevice, SensorEntity):
         if self._device.secondary_temperature is not None:
             attr[ATTR_TEMPERATURE] = self._device.secondary_temperature
 
-        if self._device.type in Consumption.ZHATYPE:
+        if isinstance(self._device, Consumption):
             attr[ATTR_POWER] = self._device.power
 
-        elif self._device.type in Daylight.ZHATYPE:
+        elif isinstance(self._device, Daylight):
             attr[ATTR_DAYLIGHT] = self._device.daylight
 
-        elif self._device.type in LightLevel.ZHATYPE:
+        elif isinstance(self._device, LightLevel):
 
             if self._device.dark is not None:
                 attr[ATTR_DARK] = self._device.dark
@@ -195,7 +229,7 @@ class DeconzSensor(DeconzDevice, SensorEntity):
             if self._device.daylight is not None:
                 attr[ATTR_DAYLIGHT] = self._device.daylight
 
-        elif self._device.type in Power.ZHATYPE:
+        elif isinstance(self._device, Power):
             attr[ATTR_CURRENT] = self._device.current
             attr[ATTR_VOLTAGE] = self._device.voltage
 
@@ -208,16 +242,13 @@ class DeconzTemperature(DeconzDevice, SensorEntity):
     Extra temperature sensor on certain Xiaomi devices.
     """
 
-    _attr_device_class = DEVICE_CLASS_TEMPERATURE
-    _attr_state_class = STATE_CLASS_MEASUREMENT
-    _attr_unit_of_measurement = TEMP_CELSIUS
-
     TYPE = DOMAIN
 
     def __init__(self, device, gateway):
         """Initialize deCONZ temperature sensor."""
         super().__init__(device, gateway)
 
+        self.entity_description = ENTITY_DESCRIPTIONS[Temperature]
         self._attr_name = f"{self._device.name} Temperature"
 
     @property
@@ -226,14 +257,14 @@ class DeconzTemperature(DeconzDevice, SensorEntity):
         return f"{self.serial}-temperature"
 
     @callback
-    def async_update_callback(self, force_update=False):
+    def async_update_callback(self):
         """Update the sensor's state."""
         keys = {"temperature", "reachable"}
-        if force_update or self._device.changed_keys.intersection(keys):
-            super().async_update_callback(force_update=force_update)
+        if self._device.changed_keys.intersection(keys):
+            super().async_update_callback()
 
     @property
-    def state(self):
+    def native_value(self):
         """Return the state of the sensor."""
         return self._device.secondary_temperature
 
@@ -241,24 +272,21 @@ class DeconzTemperature(DeconzDevice, SensorEntity):
 class DeconzBattery(DeconzDevice, SensorEntity):
     """Battery class for when a device is only represented as an event."""
 
-    _attr_device_class = DEVICE_CLASS_BATTERY
-    _attr_state_class = STATE_CLASS_MEASUREMENT
-    _attr_unit_of_measurement = PERCENTAGE
-
     TYPE = DOMAIN
 
     def __init__(self, device, gateway):
         """Initialize deCONZ battery level sensor."""
         super().__init__(device, gateway)
 
+        self.entity_description = ENTITY_DESCRIPTIONS[Battery]
         self._attr_name = f"{self._device.name} Battery Level"
 
     @callback
-    def async_update_callback(self, force_update=False):
+    def async_update_callback(self):
         """Update the battery's state, if needed."""
         keys = {"battery", "reachable"}
-        if force_update or self._device.changed_keys.intersection(keys):
-            super().async_update_callback(force_update=force_update)
+        if self._device.changed_keys.intersection(keys):
+            super().async_update_callback()
 
     @property
     def unique_id(self):
@@ -267,7 +295,7 @@ class DeconzBattery(DeconzDevice, SensorEntity):
         Normally there should only be one battery sensor per device from deCONZ.
         With specific Danfoss devices each endpoint can report its own battery state.
         """
-        if self._device.manufacturer == "Danfoss" and self._device.modelid in [
+        if self._device.manufacturer == "Danfoss" and self._device.model_id in [
             "0x8030",
             "0x8031",
             "0x8034",
@@ -277,7 +305,7 @@ class DeconzBattery(DeconzDevice, SensorEntity):
         return f"{self.serial}-battery"
 
     @property
-    def state(self):
+    def native_value(self):
         """Return the state of the battery."""
         return self._device.battery
 
@@ -286,7 +314,7 @@ class DeconzBattery(DeconzDevice, SensorEntity):
         """Return the state attributes of the battery."""
         attr = {}
 
-        if self._device.type in Switch.ZHATYPE:
+        if isinstance(self._device, Switch):
             for event in self.gateway.events:
                 if self._device == event.device:
                     attr[ATTR_EVENT_ID] = event.event_id
@@ -311,12 +339,12 @@ class DeconzSensorStateTracker:
         self.sensor = None
 
     @callback
-    def async_update_callback(self, ignore_update=False):
+    def async_update_callback(self):
         """Sensor state updated."""
         if "battery" in self.sensor.changed_keys:
             async_dispatcher_send(
                 self.gateway.hass,
-                self.gateway.async_signal_new_device(NEW_SENSOR),
+                self.gateway.signal_new_sensor,
                 [self.sensor],
             )
 
