@@ -2,6 +2,8 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Awaitable
+from typing import cast
 
 from aioguardian import Client
 
@@ -9,7 +11,7 @@ from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_ATTRIBUTION, CONF_IP_ADDRESS, CONF_PORT
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_send
-from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.entity import DeviceInfo, EntityDescription
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
@@ -25,40 +27,26 @@ from .const import (
     CONF_UID,
     DATA_CLIENT,
     DATA_COORDINATOR,
+    DATA_COORDINATOR_PAIRED_SENSOR,
     DATA_PAIRED_SENSOR_MANAGER,
-    DATA_UNSUB_DISPATCHER_CONNECT,
     DOMAIN,
     LOGGER,
     SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED,
 )
 from .util import GuardianDataUpdateCoordinator
 
-DATA_LAST_SENSOR_PAIR_DUMP = "last_sensor_pair_dump"
-
 PLATFORMS = ["binary_sensor", "sensor", "switch"]
 
 
-async def async_setup(hass: HomeAssistant, config: dict) -> bool:
-    """Set up the Elexa Guardian component."""
-    hass.data[DOMAIN] = {
-        DATA_CLIENT: {},
-        DATA_COORDINATOR: {},
-        DATA_LAST_SENSOR_PAIR_DUMP: {},
-        DATA_PAIRED_SENSOR_MANAGER: {},
-        DATA_UNSUB_DISPATCHER_CONNECT: {},
-    }
-    return True
-
-
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Elexa Guardian from a config entry."""
-    client = hass.data[DOMAIN][DATA_CLIENT][entry.entry_id] = Client(
-        entry.data[CONF_IP_ADDRESS], port=entry.data[CONF_PORT]
-    )
-    hass.data[DOMAIN][DATA_COORDINATOR][entry.entry_id] = {
-        API_SENSOR_PAIRED_SENSOR_STATUS: {}
+    hass.data.setdefault(DOMAIN, {})
+    hass.data[DOMAIN][entry.entry_id] = {
+        DATA_COORDINATOR: {},
+        DATA_COORDINATOR_PAIRED_SENSOR: {},
     }
-    hass.data[DOMAIN][DATA_UNSUB_DISPATCHER_CONNECT][entry.entry_id] = []
+
+    client = Client(entry.data[CONF_IP_ADDRESS], port=entry.data[CONF_PORT])
 
     # The valve controller's UDP-based API can't handle concurrent requests very well,
     # so we use a lock to ensure that only one API request is reaching it at a time:
@@ -66,14 +54,14 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
 
     # Set up DataUpdateCoordinators for the valve controller:
     init_valve_controller_tasks = []
-    for api, api_coro in [
+    for api, api_coro in (
         (API_SENSOR_PAIR_DUMP, client.sensor.pair_dump),
         (API_SYSTEM_DIAGNOSTICS, client.system.diagnostics),
         (API_SYSTEM_ONBOARD_SENSOR_STATUS, client.system.onboard_sensor_status),
         (API_VALVE_STATUS, client.valve.status),
         (API_WIFI_STATUS, client.wifi.status),
-    ]:
-        coordinator = hass.data[DOMAIN][DATA_COORDINATOR][entry.entry_id][
+    ):
+        coordinator = hass.data[DOMAIN][entry.entry_id][DATA_COORDINATOR][
             api
         ] = GuardianDataUpdateCoordinator(
             hass,
@@ -86,22 +74,23 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
         init_valve_controller_tasks.append(coordinator.async_refresh())
 
     await asyncio.gather(*init_valve_controller_tasks)
+    hass.data[DOMAIN][entry.entry_id][DATA_CLIENT] = client
 
     # Set up an object to evaluate each batch of paired sensor UIDs and add/remove
     # devices as appropriate:
-    paired_sensor_manager = hass.data[DOMAIN][DATA_PAIRED_SENSOR_MANAGER][
-        entry.entry_id
+    paired_sensor_manager = hass.data[DOMAIN][entry.entry_id][
+        DATA_PAIRED_SENSOR_MANAGER
     ] = PairedSensorManager(hass, entry, client, api_lock)
     await paired_sensor_manager.async_process_latest_paired_sensor_uids()
 
     @callback
-    def async_process_paired_sensor_uids():
+    def async_process_paired_sensor_uids() -> None:
         """Define a callback for when new paired sensor data is received."""
         hass.async_create_task(
             paired_sensor_manager.async_process_latest_paired_sensor_uids()
         )
 
-    hass.data[DOMAIN][DATA_COORDINATOR][entry.entry_id][
+    hass.data[DOMAIN][entry.entry_id][DATA_COORDINATOR][
         API_SENSOR_PAIR_DUMP
     ].async_add_listener(async_process_paired_sensor_uids)
 
@@ -115,12 +104,7 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
     unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
-        hass.data[DOMAIN][DATA_CLIENT].pop(entry.entry_id)
-        hass.data[DOMAIN][DATA_COORDINATOR].pop(entry.entry_id)
-        hass.data[DOMAIN][DATA_LAST_SENSOR_PAIR_DUMP].pop(entry.entry_id)
-        for unsub in hass.data[DOMAIN][DATA_UNSUB_DISPATCHER_CONNECT][entry.entry_id]:
-            unsub()
-        hass.data[DOMAIN][DATA_UNSUB_DISPATCHER_CONNECT].pop(entry.entry_id)
+        hass.data[DOMAIN].pop(entry.entry_id)
 
     return unload_ok
 
@@ -140,8 +124,7 @@ class PairedSensorManager:
         self._client = client
         self._entry = entry
         self._hass = hass
-        self._listeners = []
-        self._paired_uids = set()
+        self._paired_uids: set[str] = set()
 
     async def async_pair_sensor(self, uid: str) -> None:
         """Add a new paired sensor coordinator."""
@@ -149,13 +132,15 @@ class PairedSensorManager:
 
         self._paired_uids.add(uid)
 
-        coordinator = self._hass.data[DOMAIN][DATA_COORDINATOR][self._entry.entry_id][
-            API_SENSOR_PAIRED_SENSOR_STATUS
+        coordinator = self._hass.data[DOMAIN][self._entry.entry_id][
+            DATA_COORDINATOR_PAIRED_SENSOR
         ][uid] = GuardianDataUpdateCoordinator(
             self._hass,
             client=self._client,
             api_name=f"{API_SENSOR_PAIRED_SENSOR_STATUS}_{uid}",
-            api_coro=lambda: self._client.sensor.paired_sensor_status(uid),
+            api_coro=lambda: cast(
+                Awaitable, self._client.sensor.paired_sensor_status(uid)
+            ),
             api_lock=self._api_lock,
             valve_controller_uid=self._entry.data[CONF_UID],
         )
@@ -171,7 +156,7 @@ class PairedSensorManager:
         """Process a list of new UIDs."""
         try:
             uids = set(
-                self._hass.data[DOMAIN][DATA_COORDINATOR][self._entry.entry_id][
+                self._hass.data[DOMAIN][self._entry.entry_id][DATA_COORDINATOR][
                     API_SENSOR_PAIR_DUMP
                 ].data["paired_uids"]
             )
@@ -198,8 +183,8 @@ class PairedSensorManager:
 
         # Clear out objects related to this paired sensor:
         self._paired_uids.remove(uid)
-        self._hass.data[DOMAIN][DATA_COORDINATOR][self._entry.entry_id][
-            API_SENSOR_PAIRED_SENSOR_STATUS
+        self._hass.data[DOMAIN][self._entry.entry_id][
+            DATA_COORDINATOR_PAIRED_SENSOR
         ].pop(uid)
 
         # Remove the paired sensor device from the device registry (which will
@@ -215,52 +200,22 @@ class GuardianEntity(CoordinatorEntity):
     """Define a base Guardian entity."""
 
     def __init__(  # pylint: disable=super-init-not-called
-        self, entry: ConfigEntry, kind: str, name: str, device_class: str, icon: str
+        self, entry: ConfigEntry, description: EntityDescription
     ) -> None:
         """Initialize."""
-        self._attrs = {ATTR_ATTRIBUTION: "Data provided by Elexa"}
-        self._available = True
+        self._attr_device_info = DeviceInfo(manufacturer="Elexa")
+        self._attr_extra_state_attributes = {ATTR_ATTRIBUTION: "Data provided by Elexa"}
         self._entry = entry
-        self._device_class = device_class
-        self._device_info = {"manufacturer": "Elexa"}
-        self._icon = icon
-        self._kind = kind
-        self._name = name
-
-    @property
-    def device_class(self) -> str:
-        """Return the device class."""
-        return self._device_class
-
-    @property
-    def device_info(self) -> DeviceInfo:
-        """Return device registry information for this entity."""
-        return self._device_info
-
-    @property
-    def extra_state_attributes(self) -> dict:
-        """Return the state attributes."""
-        return self._attrs
-
-    @property
-    def icon(self) -> str:
-        """Return the icon."""
-        return self._icon
+        self.entity_description = description
 
     @callback
-    def _async_update_from_latest_data(self):
+    def _async_update_from_latest_data(self) -> None:
         """Update the entity.
 
         This should be extended by Guardian platforms.
         """
         raise NotImplementedError
 
-    @callback
-    def _async_update_state_callback(self):
-        """Update the entity's state."""
-        self._async_update_from_latest_data()
-        self.async_write_ha_state()
-
 
 class PairedSensorEntity(GuardianEntity):
     """Define a Guardian paired sensor entity."""
@@ -269,32 +224,23 @@ class PairedSensorEntity(GuardianEntity):
         self,
         entry: ConfigEntry,
         coordinator: DataUpdateCoordinator,
-        kind: str,
-        name: str,
-        device_class: str,
-        icon: str,
+        description: EntityDescription,
     ) -> None:
         """Initialize."""
-        super().__init__(entry, kind, name, device_class, icon)
+        super().__init__(entry, description)
 
+        paired_sensor_uid = coordinator.data["uid"]
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, paired_sensor_uid)},
+            name=f"Guardian Paired Sensor {paired_sensor_uid}",
+            via_device=(DOMAIN, entry.data[CONF_UID]),
+        )
+        self._attr_name = (
+            f"Guardian Paired Sensor {paired_sensor_uid}: {description.name}"
+        )
+        self._attr_unique_id = f"{paired_sensor_uid}_{description.key}"
         self.coordinator = coordinator
 
-        self._paired_sensor_uid = coordinator.data["uid"]
-
-        self._device_info["identifiers"] = {(DOMAIN, self._paired_sensor_uid)}
-        self._device_info["name"] = f"Guardian Paired Sensor {self._paired_sensor_uid}"
-        self._device_info["via_device"] = (DOMAIN, self._entry.data[CONF_UID])
-
-    @property
-    def name(self) -> str:
-        """Return the name of the entity."""
-        return f"Guardian Paired Sensor {self._paired_sensor_uid}: {self._name}"
-
-    @property
-    def unique_id(self):
-        """Return the unique ID of the entity."""
-        return f"{self._paired_sensor_uid}_{self._kind}"
-
     async def async_added_to_hass(self) -> None:
         """Perform tasks when the entity is added."""
         self._async_update_from_latest_data()
@@ -307,40 +253,29 @@ class ValveControllerEntity(GuardianEntity):
         self,
         entry: ConfigEntry,
         coordinators: dict[str, DataUpdateCoordinator],
-        kind: str,
-        name: str,
-        device_class: str,
-        icon: str,
+        description: EntityDescription,
     ) -> None:
         """Initialize."""
-        super().__init__(entry, kind, name, device_class, icon)
+        super().__init__(entry, description)
 
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, entry.data[CONF_UID])},
+            model=coordinators[API_SYSTEM_DIAGNOSTICS].data["firmware"],
+            name=f"Guardian Valve Controller {entry.data[CONF_UID]}",
+        )
+        self._attr_name = f"Guardian {entry.data[CONF_UID]}: {description.name}"
+        self._attr_unique_id = f"{entry.data[CONF_UID]}_{description.key}"
         self.coordinators = coordinators
 
-        self._device_info["identifiers"] = {(DOMAIN, self._entry.data[CONF_UID])}
-        self._device_info[
-            "name"
-        ] = f"Guardian Valve Controller {self._entry.data[CONF_UID]}"
-        self._device_info["model"] = self.coordinators[API_SYSTEM_DIAGNOSTICS].data[
-            "firmware"
-        ]
-
     @property
-    def availabile(self) -> bool:
+    def available(self) -> bool:
         """Return if entity is available."""
-        return any(coordinator.last_update_success for coordinator in self.coordinators)
+        return any(
+            coordinator.last_update_success
+            for coordinator in self.coordinators.values()
+        )
 
-    @property
-    def name(self) -> str:
-        """Return the name of the entity."""
-        return f"Guardian {self._entry.data[CONF_UID]}: {self._name}"
-
-    @property
-    def unique_id(self):
-        """Return the unique ID of the entity."""
-        return f"{self._entry.data[CONF_UID]}_{self._kind}"
-
-    async def _async_continue_entity_setup(self):
+    async def _async_continue_entity_setup(self) -> None:
         """Perform additional, internal tasks when the entity is about to be added.
 
         This should be extended by Guardian platforms.
@@ -350,9 +285,14 @@ class ValveControllerEntity(GuardianEntity):
     @callback
     def async_add_coordinator_update_listener(self, api: str) -> None:
         """Add a listener to a DataUpdateCoordinator based on the API referenced."""
-        self.async_on_remove(
-            self.coordinators[api].async_add_listener(self._async_update_state_callback)
-        )
+
+        @callback
+        def update() -> None:
+            """Update the entity's state."""
+            self._async_update_from_latest_data()
+            self.async_write_ha_state()
+
+        self.async_on_remove(self.coordinators[api].async_add_listener(update))
 
     async def async_added_to_hass(self) -> None:
         """Perform tasks when the entity is added."""
@@ -365,12 +305,12 @@ class ValveControllerEntity(GuardianEntity):
 
         Only used by the generic entity update service.
         """
-
         # Ignore manual update requests if the entity is disabled
         if not self.enabled:
             return
 
         refresh_tasks = [
-            coordinator.async_request_refresh() for coordinator in self.coordinators
+            coordinator.async_request_refresh()
+            for coordinator in self.coordinators.values()
         ]
         await asyncio.gather(*refresh_tasks)
