@@ -1,81 +1,89 @@
 """Support for views."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import Awaitable, Callable
+from http import HTTPStatus
 import json
 import logging
-from typing import List, Optional
+from typing import Any
 
 from aiohttp import web
+from aiohttp.typedefs import LooseHeaders
 from aiohttp.web_exceptions import (
     HTTPBadRequest,
     HTTPInternalServerError,
     HTTPUnauthorized,
 )
+from aiohttp.web_urldispatcher import AbstractRoute
 import voluptuous as vol
 
 from homeassistant import exceptions
-from homeassistant.const import CONTENT_TYPE_JSON, HTTP_OK
+from homeassistant.const import CONTENT_TYPE_JSON
 from homeassistant.core import Context, is_callback
 from homeassistant.helpers.json import JSONEncoder
 
-from .const import KEY_AUTHENTICATED, KEY_HASS, KEY_REAL_IP
+from .const import KEY_AUTHENTICATED, KEY_HASS
 
 _LOGGER = logging.getLogger(__name__)
 
 
-# mypy: allow-untyped-defs, no-check-untyped-defs
-
-
 class HomeAssistantView:
     """Base view for all views."""
 
-    url: Optional[str] = None
-    extra_urls: List[str] = []
+    url: str | None = None
+    extra_urls: list[str] = []
     # Views inheriting from this class can override this
     requires_auth = True
     cors_allowed = False
 
     @staticmethod
-    def context(request):
+    def context(request: web.Request) -> Context:
         """Generate a context from a request."""
-        user = request.get("hass_user")
-        if user is None:
+        if (user := request.get("hass_user")) is None:
             return Context()
 
         return Context(user_id=user.id)
 
     @staticmethod
-    def json(result, status_code=HTTP_OK, headers=None):
+    def json(
+        result: Any,
+        status_code: HTTPStatus | int = HTTPStatus.OK,
+        headers: LooseHeaders | None = None,
+    ) -> web.Response:
         """Return a JSON response."""
         try:
-            msg = json.dumps(
-                result, sort_keys=True, cls=JSONEncoder, allow_nan=False
-            ).encode("UTF-8")
+            msg = json.dumps(result, cls=JSONEncoder, allow_nan=False).encode("UTF-8")
         except (ValueError, TypeError) as err:
             _LOGGER.error("Unable to serialize to JSON: %s\n%s", err, result)
-            raise HTTPInternalServerError
+            raise HTTPInternalServerError from err
         response = web.Response(
             body=msg,
             content_type=CONTENT_TYPE_JSON,
-            status=status_code,
+            status=int(status_code),
             headers=headers,
         )
         response.enable_compression()
         return response
 
     def json_message(
-        self, message, status_code=HTTP_OK, message_code=None, headers=None
-    ):
+        self,
+        message: str,
+        status_code: HTTPStatus | int = HTTPStatus.OK,
+        message_code: str | None = None,
+        headers: LooseHeaders | None = None,
+    ) -> web.Response:
         """Return a JSON message response."""
         data = {"message": message}
         if message_code is not None:
             data["code"] = message_code
         return self.json(data, status_code, headers=headers)
 
-    def register(self, app, router):
+    def register(self, app: web.Application, router: web.UrlDispatcher) -> None:
         """Register the view with a router."""
         assert self.url is not None, "No url set for view"
         urls = [self.url] + self.extra_urls
-        routes = []
+        routes: list[AbstractRoute] = []
 
         for method in ("get", "post", "delete", "put", "patch", "head", "options"):
             handler = getattr(self, method, None)
@@ -95,16 +103,18 @@ class HomeAssistantView:
             app["allow_cors"](route)
 
 
-def request_handler_factory(view, handler):
+def request_handler_factory(
+    view: HomeAssistantView, handler: Callable
+) -> Callable[[web.Request], Awaitable[web.StreamResponse]]:
     """Wrap the handler classes."""
     assert asyncio.iscoroutinefunction(handler) or is_callback(
         handler
     ), "Handler should be a coroutine or a callback."
 
-    async def handle(request):
+    async def handle(request: web.Request) -> web.StreamResponse:
         """Handle incoming request."""
-        if not request.app[KEY_HASS].is_running:
-            return web.Response(status=503)
+        if request.app[KEY_HASS].is_stopping:
+            return web.Response(status=HTTPStatus.SERVICE_UNAVAILABLE)
 
         authenticated = request.get(KEY_AUTHENTICATED, False)
 
@@ -114,7 +124,7 @@ def request_handler_factory(view, handler):
         _LOGGER.debug(
             "Serving %s to %s (auth: %s)",
             request.path,
-            request.get(KEY_REAL_IP),
+            request.remote,
             authenticated,
         )
 
@@ -123,31 +133,33 @@ def request_handler_factory(view, handler):
 
             if asyncio.iscoroutine(result):
                 result = await result
-        except vol.Invalid:
-            raise HTTPBadRequest()
-        except exceptions.ServiceNotFound:
-            raise HTTPInternalServerError()
-        except exceptions.Unauthorized:
-            raise HTTPUnauthorized()
+        except vol.Invalid as err:
+            raise HTTPBadRequest() from err
+        except exceptions.ServiceNotFound as err:
+            raise HTTPInternalServerError() from err
+        except exceptions.Unauthorized as err:
+            raise HTTPUnauthorized() from err
 
         if isinstance(result, web.StreamResponse):
             # The method handler returned a ready-made Response, how nice of it
             return result
 
-        status_code = HTTP_OK
+        status_code = HTTPStatus.OK
 
         if isinstance(result, tuple):
             result, status_code = result
 
-        if isinstance(result, str):
-            result = result.encode("utf-8")
+        if isinstance(result, bytes):
+            bresult = result
+        elif isinstance(result, str):
+            bresult = result.encode("utf-8")
         elif result is None:
-            result = b""
-        elif not isinstance(result, bytes):
+            bresult = b""
+        else:
             assert (
                 False
-            ), f"Result should be None, string, bytes or Response. Got: {result}"
+            ), f"Result should be None, string, bytes or StreamResponse. Got: {result}"
 
-        return web.Response(body=result, status=status_code)
+        return web.Response(body=bresult, status=status_code)
 
     return handle
