@@ -2,9 +2,10 @@
 from __future__ import annotations
 
 from bisect import bisect
+from collections.abc import Callable
 import logging
 import re
-from typing import Callable, NamedTuple
+from typing import NamedTuple
 
 import attr
 
@@ -12,19 +13,22 @@ from homeassistant.components.sensor import (
     DEVICE_CLASS_BATTERY,
     DEVICE_CLASS_SIGNAL_STRENGTH,
     DOMAIN as SENSOR_DOMAIN,
+    STATE_CLASS_MEASUREMENT,
+    STATE_CLASS_TOTAL_INCREASING,
     SensorEntity,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
-    CONF_URL,
     DATA_BYTES,
     DATA_RATE_BYTES_PER_SECOND,
+    FREQUENCY_MEGAHERTZ,
     PERCENTAGE,
     STATE_UNKNOWN,
     TIME_SECONDS,
 )
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity import Entity
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.typing import StateType
 
 from . import HuaweiLteBaseEntity
@@ -52,6 +56,7 @@ class SensorMeta(NamedTuple):
     device_class: str | None = None
     icon: str | Callable[[StateType], str] | None = None
     unit: str | None = None
+    state_class: str | None = None
     enabled_default: bool = False
     include: re.Pattern[str] | None = None
     exclude: re.Pattern[str] | None = None
@@ -121,6 +126,7 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
             "mdi:signal-cellular-2",
             "mdi:signal-cellular-3",
         )[bisect((-11, -8, -5), x if x is not None else -1000)],
+        state_class=STATE_CLASS_MEASUREMENT,
         enabled_default=True,
     ),
     (KEY_DEVICE_SIGNAL, "rsrp"): SensorMeta(
@@ -133,6 +139,7 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
             "mdi:signal-cellular-2",
             "mdi:signal-cellular-3",
         )[bisect((-110, -95, -80), x if x is not None else -1000)],
+        state_class=STATE_CLASS_MEASUREMENT,
         enabled_default=True,
     ),
     (KEY_DEVICE_SIGNAL, "rssi"): SensorMeta(
@@ -145,6 +152,7 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
             "mdi:signal-cellular-2",
             "mdi:signal-cellular-3",
         )[bisect((-80, -70, -60), x if x is not None else -1000)],
+        state_class=STATE_CLASS_MEASUREMENT,
         enabled_default=True,
     ),
     (KEY_DEVICE_SIGNAL, "sinr"): SensorMeta(
@@ -157,6 +165,7 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
             "mdi:signal-cellular-2",
             "mdi:signal-cellular-3",
         )[bisect((0, 5, 10), x if x is not None else -1000)],
+        state_class=STATE_CLASS_MEASUREMENT,
         enabled_default=True,
     ),
     (KEY_DEVICE_SIGNAL, "rscp"): SensorMeta(
@@ -169,6 +178,7 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
             "mdi:signal-cellular-2",
             "mdi:signal-cellular-3",
         )[bisect((-95, -85, -75), x if x is not None else -1000)],
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     (KEY_DEVICE_SIGNAL, "ecio"): SensorMeta(
         name="EC/IO",
@@ -180,6 +190,7 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
             "mdi:signal-cellular-2",
             "mdi:signal-cellular-3",
         )[bisect((-20, -10, -6), x if x is not None else -1000)],
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     (KEY_DEVICE_SIGNAL, "transmode"): SensorMeta(name="Transmission mode"),
     (KEY_DEVICE_SIGNAL, "cqi0"): SensorMeta(
@@ -192,11 +203,17 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
     ),
     (KEY_DEVICE_SIGNAL, "ltedlfreq"): SensorMeta(
         name="Downlink frequency",
-        formatter=lambda x: (round(int(x) / 10), "MHz"),
+        formatter=lambda x: (
+            round(int(x) / 10) if x is not None else None,
+            FREQUENCY_MEGAHERTZ,
+        ),
     ),
     (KEY_DEVICE_SIGNAL, "lteulfreq"): SensorMeta(
         name="Uplink frequency",
-        formatter=lambda x: (round(int(x) / 10), "MHz"),
+        formatter=lambda x: (
+            round(int(x) / 10) if x is not None else None,
+            FREQUENCY_MEGAHERTZ,
+        ),
     ),
     KEY_MONITORING_CHECK_NOTIFICATIONS: SensorMeta(
         exclude=re.compile(
@@ -211,10 +228,16 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
         exclude=re.compile(r"^month(duration|lastcleartime)$", re.IGNORECASE)
     ),
     (KEY_MONITORING_MONTH_STATISTICS, "CurrentMonthDownload"): SensorMeta(
-        name="Current month download", unit=DATA_BYTES, icon="mdi:download"
+        name="Current month download",
+        unit=DATA_BYTES,
+        icon="mdi:download",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     (KEY_MONITORING_MONTH_STATISTICS, "CurrentMonthUpload"): SensorMeta(
-        name="Current month upload", unit=DATA_BYTES, icon="mdi:upload"
+        name="Current month upload",
+        unit=DATA_BYTES,
+        icon="mdi:upload",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     KEY_MONITORING_STATUS: SensorMeta(
         include=re.compile(
@@ -226,9 +249,12 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
         name="Battery",
         device_class=DEVICE_CLASS_BATTERY,
         unit=PERCENTAGE,
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     (KEY_MONITORING_STATUS, "CurrentWifiUser"): SensorMeta(
-        name="WiFi clients connected", icon="mdi:wifi"
+        name="WiFi clients connected",
+        icon="mdi:wifi",
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     (KEY_MONITORING_STATUS, "PrimaryDns"): SensorMeta(
         name="Primary DNS server", icon="mdi:ip"
@@ -249,29 +275,46 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
         name="Current connection duration", unit=TIME_SECONDS, icon="mdi:timer-outline"
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "CurrentDownload"): SensorMeta(
-        name="Current connection download", unit=DATA_BYTES, icon="mdi:download"
+        name="Current connection download",
+        unit=DATA_BYTES,
+        icon="mdi:download",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "CurrentDownloadRate"): SensorMeta(
         name="Current download rate",
         unit=DATA_RATE_BYTES_PER_SECOND,
         icon="mdi:download",
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "CurrentUpload"): SensorMeta(
-        name="Current connection upload", unit=DATA_BYTES, icon="mdi:upload"
+        name="Current connection upload",
+        unit=DATA_BYTES,
+        icon="mdi:upload",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "CurrentUploadRate"): SensorMeta(
         name="Current upload rate",
         unit=DATA_RATE_BYTES_PER_SECOND,
         icon="mdi:upload",
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "TotalConnectTime"): SensorMeta(
-        name="Total connected duration", unit=TIME_SECONDS, icon="mdi:timer-outline"
+        name="Total connected duration",
+        unit=TIME_SECONDS,
+        icon="mdi:timer-outline",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "TotalDownload"): SensorMeta(
-        name="Total download", unit=DATA_BYTES, icon="mdi:download"
+        name="Total download",
+        unit=DATA_BYTES,
+        icon="mdi:download",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     (KEY_MONITORING_TRAFFIC_STATISTICS, "TotalUpload"): SensorMeta(
-        name="Total upload", unit=DATA_BYTES, icon="mdi:upload"
+        name="Total upload",
+        unit=DATA_BYTES,
+        icon="mdi:upload",
+        state_class=STATE_CLASS_TOTAL_INCREASING,
     ),
     KEY_NET_CURRENT_PLMN: SensorMeta(
         exclude=re.compile(r"^(Rat|ShortName|Spn)$", re.IGNORECASE)
@@ -356,10 +399,10 @@ SENSOR_META: dict[str | tuple[str, str], SensorMeta] = {
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
-    async_add_entities: Callable[[list[Entity], bool], None],
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up from config entry."""
-    router = hass.data[DOMAIN].routers[config_entry.data[CONF_URL]]
+    router = hass.data[DOMAIN].routers[config_entry.unique_id]
     sensors: list[Entity] = []
     for key in SENSOR_KEYS:
         if not (items := router.data.get(key)):
@@ -425,7 +468,7 @@ class HuaweiLteSensor(HuaweiLteBaseEntity, SensorEntity):
         return f"{self.key}.{self.item}"
 
     @property
-    def state(self) -> StateType:
+    def native_value(self) -> StateType:
         """Return sensor state."""
         return self._state
 
@@ -435,7 +478,7 @@ class HuaweiLteSensor(HuaweiLteBaseEntity, SensorEntity):
         return self.meta.device_class
 
     @property
-    def unit_of_measurement(self) -> str | None:
+    def native_unit_of_measurement(self) -> str | None:
         """Return sensor's unit of measurement."""
         return self.meta.unit or self._unit
 
@@ -447,6 +490,11 @@ class HuaweiLteSensor(HuaweiLteBaseEntity, SensorEntity):
             return icon(self.state)
         return icon
 
+    @property
+    def state_class(self) -> str | None:
+        """Return sensor state class."""
+        return self.meta.state_class
+
     @property
     def entity_registry_enabled_default(self) -> bool:
         """Return if the entity should be enabled when first added to the entity registry."""
