@@ -1,39 +1,46 @@
 """Sensor to collect the reference daily prices of electricity ('PVPC') in Spain."""
+from __future__ import annotations
+
 import logging
 from random import randint
-from typing import Optional
+from typing import Any
 
 from aiopvpc import PVPCData
 
-from homeassistant import config_entries
-from homeassistant.const import CONF_NAME
+from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT, SensorEntity
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import CONF_NAME, CURRENCY_EURO, ENERGY_KILO_WATT_HOUR
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_call_later, async_track_time_change
 from homeassistant.helpers.restore_state import RestoreEntity
 import homeassistant.util.dt as dt_util
 
-from .const import ATTR_TARIFF
+from .const import ATTR_POWER, ATTR_POWER_P3, ATTR_TARIFF
 
 _LOGGER = logging.getLogger(__name__)
 
 ATTR_PRICE = "price"
 ICON = "mdi:currency-eur"
-UNIT = "€/kWh"
+UNIT = f"{CURRENCY_EURO}/{ENERGY_KILO_WATT_HOUR}"
 
 _DEFAULT_TIMEOUT = 10
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, config_entry: config_entries.ConfigEntry, async_add_entities
-):
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the electricity price sensor from config_entry."""
     name = config_entry.data[CONF_NAME]
     pvpc_data_handler = PVPCData(
         tariff=config_entry.data[ATTR_TARIFF],
+        power=config_entry.data[ATTR_POWER],
+        power_valley=config_entry.data[ATTR_POWER_P3],
         local_timezone=hass.config.time_zone,
         websession=async_get_clientsession(hass),
-        logger=_LOGGER,
         timeout=_DEFAULT_TIMEOUT,
     )
     async_add_entities(
@@ -41,12 +48,13 @@ async def async_setup_entry(
     )
 
 
-class ElecPriceSensor(RestoreEntity):
+class ElecPriceSensor(RestoreEntity, SensorEntity):
     """Class to hold the prices of electricity as a sensor."""
 
-    unit_of_measurement = UNIT
-    icon = ICON
-    should_poll = False
+    _attr_icon = ICON
+    _attr_native_unit_of_measurement = UNIT
+    _attr_should_poll = False
+    _attr_state_class = STATE_CLASS_MEASUREMENT
 
     def __init__(self, name, unique_id, pvpc_data_handler):
         """Initialize the sensor object."""
@@ -55,15 +63,7 @@ class ElecPriceSensor(RestoreEntity):
         self._pvpc_data = pvpc_data_handler
         self._num_retries = 0
 
-        self._hourly_tracker = None
-        self._price_tracker = None
-
-    async def async_will_remove_from_hass(self) -> None:
-        """Cancel listeners for sensor updates."""
-        self._hourly_tracker()
-        self._price_tracker()
-
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Handle entity which will be added."""
         await super().async_added_to_hass()
         state = await self.async_get_last_state()
@@ -71,14 +71,18 @@ class ElecPriceSensor(RestoreEntity):
             self._pvpc_data.state = state.state
 
         # Update 'state' value in hour changes
-        self._hourly_tracker = async_track_time_change(
-            self.hass, self.update_current_price, second=[0], minute=[0]
+        self.async_on_remove(
+            async_track_time_change(
+                self.hass, self.update_current_price, second=[0], minute=[0]
+            )
         )
         # Update prices at random time, 2 times/hour (don't want to upset API)
         random_minute = randint(1, 29)
         mins_update = [random_minute, random_minute + 30]
-        self._price_tracker = async_track_time_change(
-            self.hass, self.async_update_prices, second=[0], minute=mins_update,
+        self.async_on_remove(
+            async_track_time_change(
+                self.hass, self.async_update_prices, second=[0], minute=mins_update
+            )
         )
         _LOGGER.debug(
             "Setup of price sensor %s (%s) with tariff '%s', "
@@ -88,21 +92,22 @@ class ElecPriceSensor(RestoreEntity):
             self._pvpc_data.tariff,
             mins_update,
         )
-        await self.async_update_prices(dt_util.utcnow())
-        self.update_current_price(dt_util.utcnow())
+        now = dt_util.utcnow()
+        await self.async_update_prices(now)
+        self.update_current_price(now)
 
     @property
-    def unique_id(self) -> Optional[str]:
+    def unique_id(self) -> str | None:
         """Return a unique ID."""
         return self._unique_id
 
     @property
-    def name(self):
+    def name(self) -> str:
         """Return the name of the sensor."""
         return self._name
 
     @property
-    def state(self):
+    def native_value(self) -> float:
         """Return the state of the sensor."""
         return self._pvpc_data.state
 
@@ -112,7 +117,7 @@ class ElecPriceSensor(RestoreEntity):
         return self._pvpc_data.state_available
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self) -> dict[str, Any]:
         """Return the state attributes."""
         return self._pvpc_data.attributes
 
