@@ -1,6 +1,7 @@
 """The Synology DSM component."""
 from __future__ import annotations
 
+from collections.abc import Callable
 from datetime import timedelta
 import logging
 from typing import Any
@@ -15,17 +16,20 @@ from synology_dsm.api.dsm.information import SynoDSMInformation
 from synology_dsm.api.dsm.network import SynoDSMNetwork
 from synology_dsm.api.storage.storage import SynoStorage
 from synology_dsm.api.surveillance_station import SynoSurveillanceStation
+from synology_dsm.api.surveillance_station.camera import SynoCamera
 from synology_dsm.exceptions import (
     SynologyDSMAPIErrorException,
+    SynologyDSMLogin2SARequiredException,
+    SynologyDSMLoginDisabledAccountException,
     SynologyDSMLoginFailedException,
+    SynologyDSMLoginInvalidException,
+    SynologyDSMLoginPermissionDeniedException,
     SynologyDSMRequestException,
 )
-import voluptuous as vol
 
-from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_ATTRIBUTION,
-    CONF_DISKS,
     CONF_HOST,
     CONF_MAC,
     CONF_PASSWORD,
@@ -37,9 +41,14 @@ from homeassistant.const import (
     CONF_VERIFY_SSL,
 )
 from homeassistant.core import HomeAssistant, ServiceCall, callback
-from homeassistant.exceptions import ConfigEntryNotReady
-from homeassistant.helpers import entity_registry
+from homeassistant.exceptions import ConfigEntryAuthFailed, ConfigEntryNotReady
+from homeassistant.helpers import device_registry, entity_registry
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.device_registry import (
+    DeviceEntry,
+    async_get_registry as get_dev_reg,
+)
+from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
@@ -49,19 +58,14 @@ from homeassistant.helpers.update_coordinator import (
 from .const import (
     CONF_DEVICE_TOKEN,
     CONF_SERIAL,
-    CONF_VOLUMES,
     COORDINATOR_CAMERAS,
     COORDINATOR_CENTRAL,
     COORDINATOR_SWITCHES,
     DEFAULT_SCAN_INTERVAL,
-    DEFAULT_USE_SSL,
     DEFAULT_VERIFY_SSL,
     DOMAIN,
-    ENTITY_CLASS,
-    ENTITY_ENABLE,
-    ENTITY_ICON,
-    ENTITY_NAME,
-    ENTITY_UNIT,
+    EXCEPTION_DETAILS,
+    EXCEPTION_UNKNOWN,
     PLATFORMS,
     SERVICE_REBOOT,
     SERVICE_SHUTDOWN,
@@ -73,25 +77,11 @@ from .const import (
     SYSTEM_LOADED,
     UNDO_UPDATE_LISTENER,
     UTILISATION_SENSORS,
+    SynologyDSMEntityDescription,
 )
 
-CONFIG_SCHEMA = vol.Schema(
-    {
-        vol.Required(CONF_HOST): cv.string,
-        vol.Optional(CONF_PORT): cv.port,
-        vol.Optional(CONF_SSL, default=DEFAULT_USE_SSL): cv.boolean,
-        vol.Optional(CONF_VERIFY_SSL, default=DEFAULT_VERIFY_SSL): cv.boolean,
-        vol.Required(CONF_USERNAME): cv.string,
-        vol.Required(CONF_PASSWORD): cv.string,
-        vol.Optional(CONF_DISKS): cv.ensure_list,
-        vol.Optional(CONF_VOLUMES): cv.ensure_list,
-    }
-)
+CONFIG_SCHEMA = cv.deprecated(DOMAIN)
 
-CONFIG_SCHEMA = vol.Schema(
-    {DOMAIN: vol.Schema(vol.All(cv.ensure_list, [CONFIG_SCHEMA]))},
-    extra=vol.ALLOW_EXTRA,
-)
 
 ATTRIBUTION = "Data provided by Synology"
 
@@ -99,42 +89,27 @@ ATTRIBUTION = "Data provided by Synology"
 _LOGGER = logging.getLogger(__name__)
 
 
-async def async_setup(hass, config):
-    """Set up Synology DSM sensors from legacy config file."""
-
-    conf = config.get(DOMAIN)
-    if conf is None:
-        return True
-
-    for dsm_conf in conf:
-        hass.async_create_task(
-            hass.config_entries.flow.async_init(
-                DOMAIN,
-                context={"source": SOURCE_IMPORT},
-                data=dsm_conf,
-            )
-        )
-
-    return True
-
-
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(  # noqa: C901
+    hass: HomeAssistant, entry: ConfigEntry
+) -> bool:
     """Set up Synology DSM sensors."""
 
     # Migrate old unique_id
     @callback
-    def _async_migrator(entity_entry: entity_registry.RegistryEntry):
+    def _async_migrator(
+        entity_entry: entity_registry.RegistryEntry,
+    ) -> dict[str, str] | None:
         """Migrate away from ID using label."""
         # Reject if new unique_id
         if "SYNO." in entity_entry.unique_id:
             return None
 
-        entries = {
-            **STORAGE_DISK_BINARY_SENSORS,
-            **STORAGE_DISK_SENSORS,
-            **STORAGE_VOL_SENSORS,
-            **UTILISATION_SENSORS,
-        }
+        entries = (
+            *STORAGE_DISK_BINARY_SENSORS,
+            *STORAGE_DISK_SENSORS,
+            *STORAGE_VOL_SENSORS,
+            *UTILISATION_SENSORS,
+        )
         infos = entity_entry.unique_id.split("_")
         serial = infos.pop(0)
         label = infos.pop(0)
@@ -148,23 +123,26 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         ):
             return None
 
-        entity_type = None
-        for entity_key, entity_attrs in entries.items():
+        entity_type: str | None = None
+        for description in entries:
             if (
                 device_id
-                and entity_attrs[ENTITY_NAME] == "Status"
+                and description.name == "Status"
                 and "Status" in entity_entry.unique_id
                 and "(Smart)" not in entity_entry.unique_id
             ):
-                if "sd" in device_id and "disk" in entity_key:
-                    entity_type = entity_key
+                if "sd" in device_id and "disk" in description.key:
+                    entity_type = description.key
                     continue
-                if "volume" in device_id and "volume" in entity_key:
-                    entity_type = entity_key
+                if "volume" in device_id and "volume" in description.key:
+                    entity_type = description.key
                     continue
 
-            if entity_attrs[ENTITY_NAME] == label:
-                entity_type = entity_key
+            if description.name == label:
+                entity_type = description.key
+
+        if entity_type is None:
+            return None
 
         new_unique_id = "_".join([serial, entity_type])
         if device_id:
@@ -179,6 +157,22 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
 
     await entity_registry.async_migrate_entries(hass, entry.entry_id, _async_migrator)
 
+    # migrate device indetifiers
+    dev_reg = await get_dev_reg(hass)
+    devices: list[DeviceEntry] = device_registry.async_entries_for_config_entry(
+        dev_reg, entry.entry_id
+    )
+    for device in devices:
+        old_identifier = list(next(iter(device.identifiers)))
+        if len(old_identifier) > 2:
+            new_identifier = {
+                (old_identifier.pop(0), "_".join([str(x) for x in old_identifier]))
+            }
+            _LOGGER.debug(
+                "migrate identifier '%s' to '%s'", device.identifiers, new_identifier
+            )
+            dev_reg.async_update_device(device.id, new_identifiers=new_identifier)
+
     # Migrate existing entry configuration
     if entry.data.get(CONF_VERIFY_SSL) is None:
         hass.config_entries.async_update_entry(
@@ -189,11 +183,25 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     api = SynoApi(hass, entry)
     try:
         await api.async_setup()
+    except (
+        SynologyDSMLogin2SARequiredException,
+        SynologyDSMLoginDisabledAccountException,
+        SynologyDSMLoginInvalidException,
+        SynologyDSMLoginPermissionDeniedException,
+    ) as err:
+        if err.args[0] and isinstance(err.args[0], dict):
+            # pylint: disable=no-member
+            details = err.args[0].get(EXCEPTION_DETAILS, EXCEPTION_UNKNOWN)
+        else:
+            details = EXCEPTION_UNKNOWN
+        raise ConfigEntryAuthFailed(f"reason: {details}") from err
     except (SynologyDSMLoginFailedException, SynologyDSMRequestException) as err:
-        _LOGGER.debug(
-            "Unable to connect to DSM '%s' during setup: %s", entry.unique_id, err
-        )
-        raise ConfigEntryNotReady from err
+        if err.args[0] and isinstance(err.args[0], dict):
+            # pylint: disable=no-member
+            details = err.args[0].get(EXCEPTION_DETAILS, EXCEPTION_UNKNOWN)
+        else:
+            details = EXCEPTION_UNKNOWN
+        raise ConfigEntryNotReady(details) from err
 
     hass.data.setdefault(DOMAIN, {})
     hass.data[DOMAIN][entry.unique_id] = {
@@ -212,7 +220,9 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
             entry, data={**entry.data, CONF_MAC: network.macs}
         )
 
-    async def async_coordinator_update_data_cameras():
+    async def async_coordinator_update_data_cameras() -> dict[
+        str, dict[str, SynoCamera]
+    ] | None:
         """Fetch all camera data from api."""
         if not hass.data[DOMAIN][entry.unique_id][SYSTEM_LOADED]:
             raise UpdateFailed("System not fully loaded")
@@ -223,7 +233,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         surveillance_station = api.surveillance_station
 
         try:
-            async with async_timeout.timeout(10):
+            async with async_timeout.timeout(30):
                 await hass.async_add_executor_job(surveillance_station.update)
         except SynologyDSMAPIErrorException as err:
             raise UpdateFailed(f"Error communicating with API: {err}") from err
@@ -234,7 +244,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
             }
         }
 
-    async def async_coordinator_update_data_central():
+    async def async_coordinator_update_data_central() -> None:
         """Fetch all device and sensor data from api."""
         try:
             await api.async_update()
@@ -242,7 +252,9 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
             raise UpdateFailed(f"Error communicating with API: {err}") from err
         return None
 
-    async def async_coordinator_update_data_switches():
+    async def async_coordinator_update_data_switches() -> dict[
+        str, dict[str, Any]
+    ] | None:
         """Fetch all switch data from api."""
         if not hass.data[DOMAIN][entry.unique_id][SYSTEM_LOADED]:
             raise UpdateFailed("System not fully loaded")
@@ -290,7 +302,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     return True
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload Synology DSM sensors."""
     unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
@@ -302,15 +314,15 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     return unload_ok
 
 
-async def _async_update_listener(hass: HomeAssistant, entry: ConfigEntry):
+async def _async_update_listener(hass: HomeAssistant, entry: ConfigEntry) -> None:
     """Handle options update."""
     await hass.config_entries.async_reload(entry.entry_id)
 
 
-async def _async_setup_services(hass: HomeAssistant):
+async def _async_setup_services(hass: HomeAssistant) -> None:
     """Service handler setup."""
 
-    async def service_handler(call: ServiceCall):
+    async def service_handler(call: ServiceCall) -> None:
         """Handle service call."""
         serial = call.data.get(CONF_SERIAL)
         dsm_devices = hass.data[DOMAIN]
@@ -346,10 +358,14 @@ async def _async_setup_services(hass: HomeAssistant):
 class SynoApi:
     """Class to interface with Synology DSM API."""
 
-    def __init__(self, hass: HomeAssistant, entry: ConfigEntry):
+    def __init__(self, hass: HomeAssistant, entry: ConfigEntry) -> None:
         """Initialize the API wrapper class."""
         self._hass = hass
         self._entry = entry
+        if entry.data.get(CONF_SSL):
+            self.config_url = f"https://{entry.data[CONF_HOST]}:{entry.data[CONF_PORT]}"
+        else:
+            self.config_url = f"http://{entry.data[CONF_HOST]}:{entry.data[CONF_PORT]}"
 
         # DSM APIs
         self.dsm: SynologyDSM = None
@@ -363,7 +379,7 @@ class SynoApi:
         self.utilisation: SynoCoreUtilization = None
 
         # Should we fetch them
-        self._fetching_entities = {}
+        self._fetching_entities: dict[str, set[str]] = {}
         self._with_information = True
         self._with_security = True
         self._with_storage = True
@@ -372,7 +388,7 @@ class SynoApi:
         self._with_upgrade = True
         self._with_utilisation = True
 
-    async def async_setup(self):
+    async def async_setup(self) -> None:
         """Start interacting with the NAS."""
         self.dsm = SynologyDSM(
             self._entry.data[CONF_HOST],
@@ -402,7 +418,7 @@ class SynoApi:
         await self.async_update()
 
     @callback
-    def subscribe(self, api_key, unique_id):
+    def subscribe(self, api_key: str, unique_id: str) -> Callable[[], None]:
         """Subscribe an entity to API fetches."""
         _LOGGER.debug("Subscribe new entity: %s", unique_id)
         if api_key not in self._fetching_entities:
@@ -420,7 +436,7 @@ class SynoApi:
         return unsubscribe
 
     @callback
-    def _async_setup_api_requests(self):
+    def _async_setup_api_requests(self) -> None:
         """Determine if we should fetch each API, if one entity needs it."""
         # Entities not added yet, fetch all
         if not self._fetching_entities:
@@ -484,7 +500,7 @@ class SynoApi:
             self.dsm.reset(self.utilisation)
             self.utilisation = None
 
-    def _fetch_device_configuration(self):
+    def _fetch_device_configuration(self) -> None:
         """Fetch initial device config."""
         self.information = self.dsm.information
         self.network = self.dsm.network
@@ -519,7 +535,7 @@ class SynoApi:
             )
             self.surveillance_station = self.dsm.surveillance_station
 
-    async def async_reboot(self):
+    async def async_reboot(self) -> None:
         """Reboot NAS."""
         try:
             await self._hass.async_add_executor_job(self.system.reboot)
@@ -530,7 +546,7 @@ class SynoApi:
             )
             _LOGGER.debug("Exception:%s", err)
 
-    async def async_shutdown(self):
+    async def async_shutdown(self) -> None:
         """Shutdown NAS."""
         try:
             await self._hass.async_add_executor_job(self.system.shutdown)
@@ -541,7 +557,7 @@ class SynoApi:
             )
             _LOGGER.debug("Exception:%s", err)
 
-    async def async_unload(self):
+    async def async_unload(self) -> None:
         """Stop interacting with the NAS and prepare for removal from hass."""
         try:
             await self._hass.async_add_executor_job(self.dsm.logout)
@@ -550,7 +566,7 @@ class SynoApi:
                 "Logout from '%s' not possible:%s", self._entry.unique_id, err
             )
 
-    async def async_update(self, now=None):
+    async def async_update(self, now: timedelta | None = None) -> None:
         """Update function for updating API information."""
         _LOGGER.debug("Start data update for '%s'", self._entry.unique_id)
         self._async_setup_api_requests()
@@ -574,70 +590,43 @@ class SynoApi:
 class SynologyDSMBaseEntity(CoordinatorEntity):
     """Representation of a Synology NAS entry."""
 
+    entity_description: SynologyDSMEntityDescription
+    unique_id: str
+    _attr_extra_state_attributes = {ATTR_ATTRIBUTION: ATTRIBUTION}
+
     def __init__(
         self,
         api: SynoApi,
-        entity_type: str,
-        entity_info: dict[str, str],
-        coordinator: DataUpdateCoordinator,
-    ):
+        coordinator: DataUpdateCoordinator[dict[str, dict[str, Any]]],
+        description: SynologyDSMEntityDescription,
+    ) -> None:
         """Initialize the Synology DSM entity."""
         super().__init__(coordinator)
+        self.entity_description = description
 
         self._api = api
-        self._api_key = entity_type.split(":")[0]
-        self.entity_type = entity_type.split(":")[-1]
-        self._name = f"{api.network.hostname} {entity_info[ENTITY_NAME]}"
-        self._class = entity_info[ENTITY_CLASS]
-        self._enable_default = entity_info[ENTITY_ENABLE]
-        self._icon = entity_info[ENTITY_ICON]
-        self._unit = entity_info[ENTITY_UNIT]
-        self._unique_id = f"{self._api.information.serial}_{entity_type}"
+        self._attr_name = f"{api.network.hostname} {description.name}"
+        self._attr_unique_id: str = (
+            f"{api.information.serial}_{description.api_key}:{description.key}"
+        )
 
     @property
-    def unique_id(self) -> str:
-        """Return a unique ID."""
-        return self._unique_id
-
-    @property
-    def name(self) -> str:
-        """Return the name."""
-        return self._name
-
-    @property
-    def icon(self) -> str:
-        """Return the icon."""
-        return self._icon
-
-    @property
-    def device_class(self) -> str:
-        """Return the class of this device."""
-        return self._class
-
-    @property
-    def extra_state_attributes(self) -> dict[str, Any]:
-        """Return the state attributes."""
-        return {ATTR_ATTRIBUTION: ATTRIBUTION}
-
-    @property
-    def device_info(self) -> dict[str, Any]:
+    def device_info(self) -> DeviceInfo:
         """Return the device information."""
-        return {
-            "identifiers": {(DOMAIN, self._api.information.serial)},
-            "name": "Synology NAS",
-            "manufacturer": "Synology",
-            "model": self._api.information.model,
-            "sw_version": self._api.information.version_string,
-        }
+        return DeviceInfo(
+            identifiers={(DOMAIN, self._api.information.serial)},
+            name="Synology NAS",
+            manufacturer="Synology",
+            model=self._api.information.model,
+            sw_version=self._api.information.version_string,
+            configuration_url=self._api.config_url,
+        )
 
-    @property
-    def entity_registry_enabled_default(self) -> bool:
-        """Return if the entity should be enabled when first added to the entity registry."""
-        return self._enable_default
-
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Register entity for updates from API."""
-        self.async_on_remove(self._api.subscribe(self._api_key, self.unique_id))
+        self.async_on_remove(
+            self._api.subscribe(self.entity_description.api_key, self.unique_id)
+        )
         await super().async_added_to_hass()
 
 
@@ -647,21 +636,20 @@ class SynologyDSMDeviceEntity(SynologyDSMBaseEntity):
     def __init__(
         self,
         api: SynoApi,
-        entity_type: str,
-        entity_info: dict[str, str],
-        coordinator: DataUpdateCoordinator,
-        device_id: str = None,
-    ):
+        coordinator: DataUpdateCoordinator[dict[str, dict[str, Any]]],
+        description: SynologyDSMEntityDescription,
+        device_id: str | None = None,
+    ) -> None:
         """Initialize the Synology DSM disk or volume entity."""
-        super().__init__(api, entity_type, entity_info, coordinator)
+        super().__init__(api, coordinator, description)
         self._device_id = device_id
-        self._device_name = None
-        self._device_manufacturer = None
-        self._device_model = None
-        self._device_firmware = None
+        self._device_name: str | None = None
+        self._device_manufacturer: str | None = None
+        self._device_model: str | None = None
+        self._device_firmware: str | None = None
         self._device_type = None
 
-        if "volume" in entity_type:
+        if "volume" in description.key:
             volume = self._api.storage.get_volume(self._device_id)
             # Volume does not have a name
             self._device_name = volume["id"].replace("_", " ").capitalize()
@@ -674,29 +662,32 @@ class SynologyDSMDeviceEntity(SynologyDSMBaseEntity):
                 .replace("raid", "RAID")
                 .replace("shr", "SHR")
             )
-        elif "disk" in entity_type:
+        elif "disk" in description.key:
             disk = self._api.storage.get_disk(self._device_id)
             self._device_name = disk["name"]
             self._device_manufacturer = disk["vendor"]
             self._device_model = disk["model"].strip()
             self._device_firmware = disk["firm"]
             self._device_type = disk["diskType"]
-        self._name = f"{self._api.network.hostname} {self._device_name} {entity_info[ENTITY_NAME]}"
-        self._unique_id += f"_{self._device_id}"
+        self._name = (
+            f"{self._api.network.hostname} {self._device_name} {description.name}"
+        )
+        self._attr_unique_id += f"_{self._device_id}"
 
     @property
     def available(self) -> bool:
         """Return True if entity is available."""
-        return bool(self._api.storage)
+        return self._api.storage  # type: ignore [no-any-return]
 
     @property
-    def device_info(self) -> dict[str, Any]:
+    def device_info(self) -> DeviceInfo:
         """Return the device information."""
-        return {
-            "identifiers": {(DOMAIN, self._api.information.serial, self._device_id)},
-            "name": f"Synology NAS ({self._device_name} - {self._device_type})",
-            "manufacturer": self._device_manufacturer,
-            "model": self._device_model,
-            "sw_version": self._device_firmware,
-            "via_device": (DOMAIN, self._api.information.serial),
-        }
+        return DeviceInfo(
+            identifiers={(DOMAIN, f"{self._api.information.serial}_{self._device_id}")},
+            name=f"Synology NAS ({self._device_name} - {self._device_type})",
+            manufacturer=self._device_manufacturer,
+            model=self._device_model,
+            sw_version=self._device_firmware,
+            via_device=(DOMAIN, self._api.information.serial),
+            configuration_url=self._api.config_url,
+        )
