@@ -5,10 +5,18 @@ import functools
 import numbers
 from typing import Any
 
+from homeassistant.components.climate.const import (
+    CURRENT_HVAC_COOL,
+    CURRENT_HVAC_FAN,
+    CURRENT_HVAC_HEAT,
+    CURRENT_HVAC_IDLE,
+    CURRENT_HVAC_OFF,
+)
 from homeassistant.components.sensor import (
     DEVICE_CLASS_BATTERY,
     DEVICE_CLASS_CO,
     DEVICE_CLASS_CO2,
+    DEVICE_CLASS_CURRENT,
     DEVICE_CLASS_HUMIDITY,
     DEVICE_CLASS_ILLUMINANCE,
     DEVICE_CLASS_POWER,
@@ -24,7 +32,10 @@ from homeassistant.const import (
     CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
     CONCENTRATION_PARTS_PER_MILLION,
     DEVICE_CLASS_ENERGY,
+    ELECTRIC_CURRENT_AMPERE,
+    ELECTRIC_POTENTIAL_VOLT,
     ENERGY_KILO_WATT_HOUR,
+    ENTITY_CATEGORY_DIAGNOSTIC,
     LIGHT_LUX,
     PERCENTAGE,
     POWER_WATT,
@@ -54,6 +65,7 @@ from .core.const import (
     CHANNEL_PRESSURE,
     CHANNEL_SMARTENERGY_METERING,
     CHANNEL_TEMPERATURE,
+    CHANNEL_THERMOSTAT,
     DATA_ZHA,
     DATA_ZHA_DISPATCHERS,
     SIGNAL_ADD_ENTITIES,
@@ -129,6 +141,24 @@ class Sensor(ZhaEntity, SensorEntity):
         super().__init__(unique_id, zha_device, channels, **kwargs)
         self._channel: ChannelType = channels[0]
 
+    @classmethod
+    def create_entity(
+        cls,
+        unique_id: str,
+        zha_device: ZhaDeviceType,
+        channels: list[ChannelType],
+        **kwargs,
+    ) -> ZhaEntity | None:
+        """Entity Factory.
+
+        Return entity if it is a supported configuration, otherwise return None
+        """
+        channel = channels[0]
+        if cls.SENSOR_ATTR in channel.cluster.unsupported_attributes:
+            return None
+
+        return cls(unique_id, zha_device, channels, **kwargs)
+
     async def async_added_to_hass(self) -> None:
         """Run when about to be added to hass."""
         await super().async_added_to_hass()
@@ -194,6 +224,23 @@ class Battery(Sensor):
     _device_class = DEVICE_CLASS_BATTERY
     _state_class = STATE_CLASS_MEASUREMENT
     _unit = PERCENTAGE
+    _attr_entity_category = ENTITY_CATEGORY_DIAGNOSTIC
+
+    @classmethod
+    def create_entity(
+        cls,
+        unique_id: str,
+        zha_device: ZhaDeviceType,
+        channels: list[ChannelType],
+        **kwargs,
+    ) -> ZhaEntity | None:
+        """Entity Factory.
+
+        Unlike any other entity, PowerConfiguration cluster may not support
+        battery_percent_remaining attribute, but zha-device-handlers takes care of it
+        so create the entity regardless
+        """
+        return cls(unique_id, zha_device, channels, **kwargs)
 
     @staticmethod
     def formatter(value: int) -> int:
@@ -220,7 +267,7 @@ class Battery(Sensor):
         return state_attrs
 
 
-@STRICT_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
+@MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
 class ElectricalMeasurement(Sensor):
     """Active power measurement."""
 
@@ -228,16 +275,32 @@ class ElectricalMeasurement(Sensor):
     _device_class = DEVICE_CLASS_POWER
     _state_class = STATE_CLASS_MEASUREMENT
     _unit = POWER_WATT
+    _div_mul_prefix = "ac_power"
 
     @property
     def should_poll(self) -> bool:
         """Return True if HA needs to poll for state changes."""
         return True
 
+    @property
+    def extra_state_attributes(self) -> dict[str, Any]:
+        """Return device state attrs for sensor."""
+        attrs = {}
+        if self._channel.measurement_type is not None:
+            attrs["measurement_type"] = self._channel.measurement_type
+
+        max_attr_name = f"{self.SENSOR_ATTR}_max"
+        if (max_v := self._channel.cluster.get(max_attr_name)) is not None:
+            attrs[max_attr_name] = str(self.formatter(max_v))
+
+        return attrs
+
     def formatter(self, value: int) -> int | float:
         """Return 'normalized' value."""
-        value = value * self._channel.multiplier / self._channel.divisor
-        if value < 100 and self._channel.divisor > 1:
+        multiplier = getattr(self._channel, f"{self._div_mul_prefix}_multiplier")
+        divisor = getattr(self._channel, f"{self._div_mul_prefix}_divisor")
+        value = float(value * multiplier) / divisor
+        if value < 100 and divisor > 1:
             return round(value, self._decimals)
         return round(value)
 
@@ -248,6 +311,36 @@ class ElectricalMeasurement(Sensor):
         await super().async_update()
 
 
+@MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
+class ElectricalMeasurementRMSCurrent(ElectricalMeasurement, id_suffix="rms_current"):
+    """RMS current measurement."""
+
+    SENSOR_ATTR = "rms_current"
+    _device_class = DEVICE_CLASS_CURRENT
+    _unit = ELECTRIC_CURRENT_AMPERE
+    _div_mul_prefix = "ac_current"
+
+    @property
+    def should_poll(self) -> bool:
+        """Poll indirectly by ElectricalMeasurementSensor."""
+        return False
+
+
+@MULTI_MATCH(channel_names=CHANNEL_ELECTRICAL_MEASUREMENT)
+class ElectricalMeasurementRMSVoltage(ElectricalMeasurement, id_suffix="rms_voltage"):
+    """RMS Voltage measurement."""
+
+    SENSOR_ATTR = "rms_voltage"
+    _device_class = DEVICE_CLASS_CURRENT
+    _unit = ELECTRIC_POTENTIAL_VOLT
+    _div_mul_prefix = "ac_voltage"
+
+    @property
+    def should_poll(self) -> bool:
+        """Poll indirectly by ElectricalMeasurementSensor."""
+        return False
+
+
 @STRICT_MATCH(generic_ids=CHANNEL_ST_HUMIDITY_CLUSTER)
 @STRICT_MATCH(channel_names=CHANNEL_HUMIDITY)
 class Humidity(Sensor):
@@ -298,24 +391,6 @@ class SmartEnergyMetering(Sensor):
         0x0C: f"MJ/{TIME_SECONDS}",
     }
 
-    @classmethod
-    def create_entity(
-        cls,
-        unique_id: str,
-        zha_device: ZhaDeviceType,
-        channels: list[ChannelType],
-        **kwargs,
-    ) -> ZhaEntity | None:
-        """Entity Factory.
-
-        Return entity if it is a supported configuration, otherwise return None
-        """
-        se_channel = channels[0]
-        if cls.SENSOR_ATTR in se_channel.cluster.unsupported_attributes:
-            return None
-
-        return cls(unique_id, zha_device, channels, **kwargs)
-
     def formatter(self, value: int) -> int | float:
         """Pass through channel formatter."""
         return self._channel.demand_formatter(value)
@@ -331,8 +406,7 @@ class SmartEnergyMetering(Sensor):
         attrs = {}
         if self._channel.device_type is not None:
             attrs["device_type"] = self._channel.device_type
-        status = self._channel.status
-        if status is not None:
+        if (status := self._channel.status) is not None:
             attrs["status"] = str(status)[len(status.__class__.__name__) + 1 :]
         return attrs
 
@@ -433,3 +507,119 @@ class FormaldehydeConcentration(Sensor):
     _decimals = 0
     _multiplier = 1e6
     _unit = CONCENTRATION_PARTS_PER_MILLION
+
+
+@MULTI_MATCH(channel_names=CHANNEL_THERMOSTAT)
+class ThermostatHVACAction(Sensor, id_suffix="hvac_action"):
+    """Thermostat HVAC action sensor."""
+
+    @classmethod
+    def create_entity(
+        cls,
+        unique_id: str,
+        zha_device: ZhaDeviceType,
+        channels: list[ChannelType],
+        **kwargs,
+    ) -> ZhaEntity | None:
+        """Entity Factory.
+
+        Return entity if it is a supported configuration, otherwise return None
+        """
+
+        return cls(unique_id, zha_device, channels, **kwargs)
+
+    @property
+    def native_value(self) -> str | None:
+        """Return the current HVAC action."""
+        if (
+            self._channel.pi_heating_demand is None
+            and self._channel.pi_cooling_demand is None
+        ):
+            return self._rm_rs_action
+        return self._pi_demand_action
+
+    @property
+    def _rm_rs_action(self) -> str | None:
+        """Return the current HVAC action based on running mode and running state."""
+
+        running_mode = self._channel.running_mode
+        if running_mode == self._channel.RunningMode.Heat:
+            return CURRENT_HVAC_HEAT
+        if running_mode == self._channel.RunningMode.Cool:
+            return CURRENT_HVAC_COOL
+
+        running_state = self._channel.running_state
+        if running_state and running_state & (
+            self._channel.RunningState.Fan_State_On
+            | self._channel.RunningState.Fan_2nd_Stage_On
+            | self._channel.RunningState.Fan_3rd_Stage_On
+        ):
+            return CURRENT_HVAC_FAN
+        if (
+            self._channel.system_mode != self._channel.SystemMode.Off
+            and running_mode == self._channel.SystemMode.Off
+        ):
+            return CURRENT_HVAC_IDLE
+        return CURRENT_HVAC_OFF
+
+    @property
+    def _pi_demand_action(self) -> str | None:
+        """Return the current HVAC action based on pi_demands."""
+
+        heating_demand = self._channel.pi_heating_demand
+        if heating_demand is not None and heating_demand > 0:
+            return CURRENT_HVAC_HEAT
+        cooling_demand = self._channel.pi_cooling_demand
+        if cooling_demand is not None and cooling_demand > 0:
+            return CURRENT_HVAC_COOL
+
+        if self._channel.system_mode != self._channel.SystemMode.Off:
+            return CURRENT_HVAC_IDLE
+        return CURRENT_HVAC_OFF
+
+    @callback
+    def async_set_state(self, *args, **kwargs) -> None:
+        """Handle state update from channel."""
+        self.async_write_ha_state()
+
+
+@MULTI_MATCH(
+    channel_names=CHANNEL_THERMOSTAT,
+    manufacturers="Zen Within",
+    stop_on_match=True,
+)
+class ZenHVACAction(ThermostatHVACAction):
+    """Zen Within Thermostat HVAC Action."""
+
+    @property
+    def _rm_rs_action(self) -> str | None:
+        """Return the current HVAC action based on running mode and running state."""
+
+        if (running_state := self._channel.running_state) is None:
+            return None
+
+        rs_heat = (
+            self._channel.RunningState.Heat_State_On
+            | self._channel.RunningState.Heat_2nd_Stage_On
+        )
+        if running_state & rs_heat:
+            return CURRENT_HVAC_HEAT
+
+        rs_cool = (
+            self._channel.RunningState.Cool_State_On
+            | self._channel.RunningState.Cool_2nd_Stage_On
+        )
+        if running_state & rs_cool:
+            return CURRENT_HVAC_COOL
+
+        running_state = self._channel.running_state
+        if running_state and running_state & (
+            self._channel.RunningState.Fan_State_On
+            | self._channel.RunningState.Fan_2nd_Stage_On
+            | self._channel.RunningState.Fan_3rd_Stage_On
+        ):
+            return CURRENT_HVAC_FAN
+
+        if self._channel.system_mode != self._channel.SystemMode.Off:
+            return CURRENT_HVAC_IDLE
+        return CURRENT_HVAC_OFF
