@@ -87,18 +87,22 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
 
     hub_is_initialized = False
 
+    def _start_hub():
+        """Start the hub socket and query status of all devices."""
+        hub.start_listen()
+        hub.query_all_status()
+
     async def startup():
         """Start hub socket after all climate entity is set up."""
         nonlocal hub_is_initialized
-        if not all([device.is_initialized for device in devices]):
+        if not all(device.is_initialized for device in devices):
             return
 
         if hub_is_initialized:
             return
 
         _LOGGER.debug("zhong_hong hub start listen event")
-        await hass.async_add_job(hub.start_listen)
-        await hass.async_add_job(hub.query_all_status)
+        await hass.async_add_executor_job(_start_hub)
         hub_is_initialized = True
 
     async_dispatcher_connect(hass, SIGNAL_DEVICE_ADDED, startup)
@@ -235,12 +239,10 @@ class ZhongHongClimate(ClimateEntity):
 
     def set_temperature(self, **kwargs):
         """Set new target temperature."""
-        temperature = kwargs.get(ATTR_TEMPERATURE)
-        if temperature is not None:
+        if (temperature := kwargs.get(ATTR_TEMPERATURE)) is not None:
             self._device.set_temperature(temperature)
 
-        operation_mode = kwargs.get(ATTR_HVAC_MODE)
-        if operation_mode is not None:
+        if (operation_mode := kwargs.get(ATTR_HVAC_MODE)) is not None:
             self.set_hvac_mode(operation_mode)
 
     def set_hvac_mode(self, hvac_mode):
