@@ -16,7 +16,9 @@ from homeassistant.components import zone as zone_cmp
 from homeassistant.components.device_automation import (
     async_get_device_automation_platform,
 )
+from homeassistant.components.sensor import DEVICE_CLASS_TIMESTAMP
 from homeassistant.const import (
+    ATTR_DEVICE_CLASS,
     ATTR_GPS_ACCURACY,
     ATTR_LATITUDE,
     ATTR_LONGITUDE,
@@ -29,6 +31,7 @@ from homeassistant.const import (
     CONF_DEVICE_ID,
     CONF_DOMAIN,
     CONF_ENTITY_ID,
+    CONF_ID,
     CONF_STATE,
     CONF_VALUE_TEMPLATE,
     CONF_WEEKDAY,
@@ -66,6 +69,8 @@ from .trace import (
     trace_stack_top,
 )
 
+# mypy: disallow-any-generics
+
 FROM_CONFIG_FORMAT = "{}_from_config"
 ASYNC_FROM_CONFIG_FORMAT = "async_{}_from_config"
 
@@ -110,7 +115,7 @@ def condition_trace_update_result(**kwargs: Any) -> None:
 
 
 @contextmanager
-def trace_condition(variables: TemplateVarsType) -> Generator:
+def trace_condition(variables: TemplateVarsType) -> Generator[TraceElement, None, None]:
     """Trace condition evaluation."""
     should_pop = True
     trace_element = trace_stack_top(trace_stack_cv)
@@ -323,9 +328,8 @@ def async_numeric_state(  # noqa: C901
 
     if isinstance(entity, str):
         entity_id = entity
-        entity = hass.states.get(entity)
 
-        if entity is None:
+        if (entity := hass.states.get(entity)) is None:
             raise ConditionErrorMessage("numeric_state", f"unknown entity {entity_id}")
     else:
         entity_id = entity.entity_id
@@ -366,8 +370,7 @@ def async_numeric_state(  # noqa: C901
 
     if below is not None:
         if isinstance(below, str):
-            below_entity = hass.states.get(below)
-            if not below_entity:
+            if not (below_entity := hass.states.get(below)):
                 raise ConditionErrorMessage(
                     "numeric_state", f"unknown 'below' entity {below}"
                 )
@@ -395,8 +398,7 @@ def async_numeric_state(  # noqa: C901
 
     if above is not None:
         if isinstance(above, str):
-            above_entity = hass.states.get(above)
-            if not above_entity:
+            if not (above_entity := hass.states.get(above)):
                 raise ConditionErrorMessage(
                     "numeric_state", f"unknown 'above' entity {above}"
                 )
@@ -492,9 +494,8 @@ def state(
 
     if isinstance(entity, str):
         entity_id = entity
-        entity = hass.states.get(entity)
 
-        if entity is None:
+        if (entity := hass.states.get(entity)) is None:
             raise ConditionErrorMessage("state", f"unknown entity {entity_id}")
     else:
         entity_id = entity.entity_id
@@ -521,8 +522,7 @@ def state(
             isinstance(req_state_value, str)
             and INPUT_ENTITY_ID.match(req_state_value) is not None
         ):
-            state_entity = hass.states.get(req_state_value)
-            if not state_entity:
+            if not (state_entity := hass.states.get(req_state_value)):
                 raise ConditionErrorMessage(
                     "state", f"the 'state' entity {req_state_value} is unavailable"
                 )
@@ -733,26 +733,50 @@ def time(
     if after is None:
         after = dt_util.dt.time(0)
     elif isinstance(after, str):
-        after_entity = hass.states.get(after)
-        if not after_entity:
+        if not (after_entity := hass.states.get(after)):
             raise ConditionErrorMessage("time", f"unknown 'after' entity {after}")
-        after = dt_util.dt.time(
-            after_entity.attributes.get("hour", 23),
-            after_entity.attributes.get("minute", 59),
-            after_entity.attributes.get("second", 59),
-        )
+        if after_entity.domain == "input_datetime":
+            after = dt_util.dt.time(
+                after_entity.attributes.get("hour", 23),
+                after_entity.attributes.get("minute", 59),
+                after_entity.attributes.get("second", 59),
+            )
+        elif after_entity.attributes.get(
+            ATTR_DEVICE_CLASS
+        ) == DEVICE_CLASS_TIMESTAMP and after_entity.state not in (
+            STATE_UNAVAILABLE,
+            STATE_UNKNOWN,
+        ):
+            after_datetime = dt_util.parse_datetime(after_entity.state)
+            if after_datetime is None:
+                return False
+            after = dt_util.as_local(after_datetime).time()
+        else:
+            return False
 
     if before is None:
         before = dt_util.dt.time(23, 59, 59, 999999)
     elif isinstance(before, str):
-        before_entity = hass.states.get(before)
-        if not before_entity:
+        if not (before_entity := hass.states.get(before)):
             raise ConditionErrorMessage("time", f"unknown 'before' entity {before}")
-        before = dt_util.dt.time(
-            before_entity.attributes.get("hour", 23),
-            before_entity.attributes.get("minute", 59),
-            before_entity.attributes.get("second", 59),
-        )
+        if before_entity.domain == "input_datetime":
+            before = dt_util.dt.time(
+                before_entity.attributes.get("hour", 23),
+                before_entity.attributes.get("minute", 59),
+                before_entity.attributes.get("second", 59),
+            )
+        elif before_entity.attributes.get(
+            ATTR_DEVICE_CLASS
+        ) == DEVICE_CLASS_TIMESTAMP and before_entity.state not in (
+            STATE_UNAVAILABLE,
+            STATE_UNKNOWN,
+        ):
+            before_timedatime = dt_util.parse_datetime(before_entity.state)
+            if before_timedatime is None:
+                return False
+            before = dt_util.as_local(before_timedatime).time()
+        else:
+            return False
 
     if after < before:
         condition_trace_update_result(after=after, now_time=now_time, before=before)
@@ -809,9 +833,8 @@ def zone(
 
     if isinstance(zone_ent, str):
         zone_ent_id = zone_ent
-        zone_ent = hass.states.get(zone_ent)
 
-        if zone_ent is None:
+        if (zone_ent := hass.states.get(zone_ent)) is None:
             raise ConditionErrorMessage("zone", f"unknown zone {zone_ent_id}")
 
     if entity is None:
@@ -819,9 +842,8 @@ def zone(
 
     if isinstance(entity, str):
         entity_id = entity
-        entity = hass.states.get(entity)
 
-        if entity is None:
+        if (entity := hass.states.get(entity)) is None:
             raise ConditionErrorMessage("zone", f"unknown entity {entity_id}")
     else:
         entity_id = entity.entity_id
@@ -902,6 +924,26 @@ async def async_device_from_config(
     )
 
 
+async def async_trigger_from_config(
+    hass: HomeAssistant, config: ConfigType, config_validation: bool = True
+) -> ConditionCheckerType:
+    """Test a trigger condition."""
+    if config_validation:
+        config = cv.TRIGGER_CONDITION_SCHEMA(config)
+    trigger_id = config[CONF_ID]
+
+    @trace_condition_function
+    def trigger_if(hass: HomeAssistant, variables: TemplateVarsType = None) -> bool:
+        """Validate trigger based if-condition."""
+        return (
+            variables is not None
+            and "trigger" in variables
+            and variables["trigger"].get("id") in trigger_id
+        )
+
+    return trigger_if
+
+
 async def async_validate_condition_config(
     hass: HomeAssistant, config: ConfigType | Template
 ) -> ConfigType | Template:
@@ -923,6 +965,8 @@ async def async_validate_condition_config(
         platform = await async_get_device_automation_platform(
             hass, config[CONF_DOMAIN], "condition"
         )
+        if hasattr(platform, "async_validate_condition_config"):
+            return await platform.async_validate_condition_config(hass, config)  # type: ignore
         return cast(ConfigType, platform.CONDITION_SCHEMA(config))  # type: ignore
 
     return config
@@ -976,9 +1020,7 @@ def async_extract_devices(config: ConfigType | Template) -> set[str]:
         if condition != "device":
             continue
 
-        device_id = config.get(CONF_DEVICE_ID)
-
-        if device_id is not None:
+        if (device_id := config.get(CONF_DEVICE_ID)) is not None:
             referenced.add(device_id)
 
     return referenced
